@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_alpha.c,v 1.1 2005/08/29 14:36:41 debug Exp $
+ *  $Id: cpu_alpha.c,v 1.3 2005/11/13 00:14:07 debug Exp $
  *
  *  Alpha CPU emulation.
  *
@@ -71,8 +71,8 @@ int alpha_cpu_new(struct cpu *cpu, struct memory *mem,
 
 	cpu->memory_rw = alpha_memory_rw;
 	cpu->update_translation_table = alpha_update_translation_table;
-	cpu->invalidate_translation_caches_paddr =
-	    alpha_invalidate_translation_caches_paddr;
+	cpu->invalidate_translation_caches =
+	    alpha_invalidate_translation_caches;
 	cpu->invalidate_code_translation = alpha_invalidate_code_translation;
 	cpu->is_32bit = 0;
 
@@ -187,32 +187,6 @@ void alpha_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 }
 
 
-/*
- *  alpha_cpu_show_full_statistics():
- *
- *  Show detailed statistics on opcode usage on each cpu.
- */
-void alpha_cpu_show_full_statistics(struct machine *m)
-{
-	fatal("alpha_cpu_show_full_statistics(): TODO\n");
-}
-
-
-/*
- *  alpha_cpu_tlbdump():
- *
- *  Called from the debugger to dump the TLB in a readable format.
- *  x is the cpu number to dump, or -1 to dump all CPUs.
- *
- *  If rawflag is nonzero, then the TLB contents isn't formated nicely,
- *  just dumped.
- */
-void alpha_cpu_tlbdump(struct machine *m, int x, int rawflag)
-{
-	fatal("alpha_cpu_tlbdump(): TODO\n");
-}
-
-
 /*
  *  alpha_cpu_interrupt():
  */
