@@ -25,16 +25,14 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_arm.c,v 1.29 2005/10/08 01:09:51 debug Exp $
+ *  $Id: cpu_arm.c,v 1.44 2005/11/19 18:53:07 debug Exp $
  *
  *  ARM CPU emulation.
  *
+ *
  *  A good source of quick info on ARM instruction encoding:
  *
  *	http://www.pinknoise.demon.co.uk/ARMinstrs/ARMinstrs.html
- *
- *  (Most "xxxx0101..." and similar strings in this file are from that URL,
- *  or from the ARM manual.)
  */
 
 #include <stdio.h>
@@ -62,8 +60,10 @@ static char *arm_dpiname[16] = ARM_DPI_NAMES;
 static int arm_dpi_uses_d[16] = { 1,1,1,1,1,1,1,1,0,0,0,0,1,1,1,1 };
 static int arm_dpi_uses_n[16] = { 1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0 };
 
-/*  Forward reference:  */
-void arm_pc_to_pointers(struct cpu *cpu);
+static int arm_exception_to_mode[N_ARM_EXCEPTIONS] = ARM_EXCEPTION_TO_MODE;
+
+/*  For quick_pc_to_pointers():  */
+#include "arm_quick_pc_to_pointers.h"
 
 
 /*
@@ -92,8 +92,8 @@ int arm_cpu_new(struct cpu *cpu, struct memory *mem,
 
 	cpu->memory_rw = arm_memory_rw;
 	cpu->update_translation_table = arm_update_translation_table;
-	cpu->invalidate_translation_caches_paddr =
-	    arm_invalidate_translation_caches_paddr;
+	cpu->invalidate_translation_caches =
+	    arm_invalidate_translation_caches;
 	cpu->invalidate_code_translation = arm_invalidate_code_translation;
 	cpu->translate_address = arm_translate_address;
 
@@ -109,8 +109,8 @@ int arm_cpu_new(struct cpu *cpu, struct memory *mem,
 		cpu->cd.arm.cpsr |= ARM_MODE_SVC32;
 		cpu->cd.arm.control |= ARM_CONTROL_S;
 	} else {
-		cpu->cd.arm.cpsr |= ARM_MODE_USR32;
-		cpu->cd.arm.control |= ARM_CONTROL_S | ARM_CONTROL_R;
+		cpu->cd.arm.cpsr |= ARM_MODE_SVC32;
+		cpu->cd.arm.control |= ARM_CONTROL_R;
 	}
 
 	/*  Only show name and caches etc for CPU nr 0:  */
@@ -140,6 +140,8 @@ int arm_cpu_new(struct cpu *cpu, struct memory *mem,
 		store_32bit_word(cpu, cpu->cd.arm.of_emul_addr, 0xef8c64be);
 	}
 
+	cpu->cd.arm.flags = cpu->cd.arm.cpsr >> 28;
+
 	return 1;
 }
 
@@ -163,6 +165,7 @@ void arm_setup_initial_translation_table(struct cpu *cpu, uint32_t ttb_addr)
 	}
 
 	cpu->cd.arm.control |= ARM_CONTROL_MMU;
+	cpu->translate_address = arm_translate_address_mmu;
 	cpu->cd.arm.dacr |= 0x00000003;
 	cpu->cd.arm.ttb = ttb_addr;
 
@@ -175,9 +178,63 @@ void arm_setup_initial_translation_table(struct cpu *cpu, uint32_t ttb_addr)
 			uint32_t addr = cpu->cd.arm.ttb +
 			    (((j << 28) + (i << 20)) >> 18);
 			uint32_t d = (1048576*i) | 0xc02;
+
+			if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
+				descr[0] = d;       descr[1] = d >> 8;
+				descr[2] = d >> 16; descr[3] = d >> 24;
+			} else {
+				descr[3] = d;       descr[2] = d >> 8;
+				descr[1] = d >> 16; descr[0] = d >> 24;
+			}
+			cpu->memory_rw(cpu, cpu->mem, addr, &descr[0],
+			    sizeof(descr), MEM_WRITE, PHYSICAL | NO_EXCEPTIONS);
+		}
+}
+
+
 /*
-d = (1048576 * (i + (j==12? 10 : j)*256)) | 2;
-*/
+ *  arm_translation_table_set_l1():
+ */
+void arm_translation_table_set_l1(struct cpu *cpu, uint32_t vaddr,
+	uint32_t paddr)
+{
+	unsigned int i, j, vhigh = vaddr >> 28, phigh = paddr >> 28;
+
+	for (i=0; i<256; i++)
+		for (j=vhigh; j<=vhigh; j++) {
+			unsigned char descr[4];
+			uint32_t addr = cpu->cd.arm.ttb +
+			    (((j << 28) + (i << 20)) >> 18);
+			uint32_t d = ((phigh << 28) + 1048576*i) | 0xc02;
+
+			if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
+				descr[0] = d;       descr[1] = d >> 8;
+				descr[2] = d >> 16; descr[3] = d >> 24;
+			} else {
+				descr[3] = d;       descr[2] = d >> 8;
+				descr[1] = d >> 16; descr[0] = d >> 24;
+			}
+			cpu->memory_rw(cpu, cpu->mem, addr, &descr[0],
+			    sizeof(descr), MEM_WRITE, PHYSICAL | NO_EXCEPTIONS);
+		}
+}
+
+
+/*
+ *  arm_translation_table_set_l1_b():
+ */
+void arm_translation_table_set_l1_b(struct cpu *cpu, uint32_t vaddr,
+	uint32_t paddr)
+{
+	unsigned int i, j, vhigh = vaddr >> 24, phigh = paddr >> 24;
+
+	for (i=0; i<16; i++)
+		for (j=vhigh; j<=vhigh; j++) {
+			unsigned char descr[4];
+			uint32_t addr = cpu->cd.arm.ttb +
+			    (((j << 24) + (i << 20)) >> 18);
+			uint32_t d = ((phigh << 24) + 1048576*i) | 0xc02;
+
 			if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
 				descr[0] = d;       descr[1] = d >> 8;
 				descr[2] = d >> 16; descr[3] = d >> 24;
@@ -267,9 +324,12 @@ void arm_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 	int mode = cpu->cd.arm.cpsr & ARM_FLAG_MODE;
 	int i, x = cpu->cpu_id;
 
+	cpu->cd.arm.cpsr &= 0x0fffffff;
+	cpu->cd.arm.cpsr |= (cpu->cd.arm.flags << 28);
+
 	if (gprs) {
 		symbol = get_symbol_name(&cpu->machine->symbol_context,
-		    cpu->cd.arm.r[ARM_PC], &offset);
+		    cpu->pc, &offset);
 		debug("cpu%i:  cpsr = ", x);
 		debug("%s%s%s%s%s%s",
 		    (cpu->cd.arm.cpsr & ARM_FLAG_N)? "N" : "n",
@@ -279,10 +339,9 @@ void arm_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 		    (cpu->cd.arm.cpsr & ARM_FLAG_I)? "I" : "i",
 		    (cpu->cd.arm.cpsr & ARM_FLAG_F)? "F" : "f");
 		if (mode < ARM_MODE_USR32)
-			debug("   pc =  0x%07x",
-			    (int)(cpu->cd.arm.r[ARM_PC] & 0x03ffffff));
+			debug("   pc =  0x%07x", (int)(cpu->pc & 0x03ffffff));
 		else
-			debug("   pc = 0x%08x", (int)cpu->cd.arm.r[ARM_PC]);
+			debug("   pc = 0x%08x", (int)cpu->pc);
 
 		debug("  <%s>\n", symbol != NULL? symbol : " no symbol ");
 
@@ -399,32 +458,6 @@ void arm_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 }
 
 
-/*
- *  arm_cpu_show_full_statistics():
- *
- *  Show detailed statistics on opcode usage on each cpu.
- */
-void arm_cpu_show_full_statistics(struct machine *m)
-{
-	fatal("arm_cpu_show_full_statistics(): TODO\n");
-}
-
-
-/*
- *  arm_cpu_tlbdump():
- *
- *  Called from the debugger to dump the TLB in a readable format.
- *  x is the cpu number to dump, or -1 to dump all CPUs.
- *
- *  If rawflag is nonzero, then the TLB contents isn't formated nicely,
- *  just dumped.
- */
-void arm_cpu_tlbdump(struct machine *m, int x, int rawflag)
-{
-	fatal("arm_cpu_tlbdump(): TODO\n");
-}
-
-
 /*
  *  arm_save_register_bank():
  */
@@ -524,7 +557,6 @@ void arm_load_register_bank(struct cpu *cpu)
  */
 void arm_exception(struct cpu *cpu, int exception_nr)
 {
-	int arm_exception_to_mode[N_ARM_EXCEPTIONS] = ARM_EXCEPTION_TO_MODE;
 	int oldmode, newmode;
 	uint32_t retaddr;
 
@@ -535,44 +567,52 @@ void arm_exception(struct cpu *cpu, int exception_nr)
 
 	retaddr = cpu->pc;
 
-	debug("[ arm_exception(): ");
+	if (!quiet_mode) {
+		debug("[ arm_exception(): ");
+		switch (exception_nr) {
+		case ARM_EXCEPTION_RESET:
+			fatal("RESET: TODO");
+			break;
+		case ARM_EXCEPTION_UND:
+			debug("UNDEFINED");
+			break;
+		case ARM_EXCEPTION_SWI:
+			debug("SWI");
+			break;
+		case ARM_EXCEPTION_PREF_ABT:
+			debug("PREFETCH ABORT");
+			break;
+		case ARM_EXCEPTION_IRQ:
+			debug("IRQ");
+			break;
+		case ARM_EXCEPTION_FIQ:
+			debug("FIQ");
+			break;
+		case ARM_EXCEPTION_DATA_ABT:
+			debug("DATA ABORT, far=0x%08x fsr=0x%02x",
+			    cpu->cd.arm.far, cpu->cd.arm.fsr);
+			break;
+		}
+		debug(" ]\n");
+	}
 
 	switch (exception_nr) {
 	case ARM_EXCEPTION_RESET:
 		cpu->running = 0;
-		fatal("RESET: TODO");
+		fatal("ARM RESET: TODO");
 		exit(1);
-	case ARM_EXCEPTION_UND:
-		debug("UNDEFINED");
-		retaddr += 4;
-		break;
-	case ARM_EXCEPTION_SWI:
-		debug("SWI");
-		retaddr += 4;
-		break;
-	case ARM_EXCEPTION_PREF_ABT:
-		debug("PREFETCH ABORT");
-		retaddr += 4;
-		break;
-	case ARM_EXCEPTION_IRQ:
-		debug("IRQ");
-		retaddr += 4;
-		break;
-	case ARM_EXCEPTION_FIQ:
-		debug("FIQ");
-		retaddr += 4;
-		break;
 	case ARM_EXCEPTION_DATA_ABT:
-		debug("DATA ABORT, far=0x%08x fsr=0x%02x",
-		    cpu->cd.arm.far, cpu->cd.arm.fsr);
-		retaddr += 8;
+		retaddr += 4;
 		break;
 	}
 
-	debug(" ]\n");
+	retaddr += 4;
 
 	arm_save_register_bank(cpu);
 
+	cpu->cd.arm.cpsr &= 0x0fffffff;
+	cpu->cd.arm.cpsr |= (cpu->cd.arm.flags << 28);
+
 	switch (arm_exception_to_mode[exception_nr]) {
 	case ARM_MODE_SVC32:
 		cpu->cd.arm.spsr_svc = cpu->cd.arm.cpsr; break;
@@ -599,12 +639,24 @@ void arm_exception(struct cpu *cpu, int exception_nr)
 	cpu->cd.arm.cpsr &= ~ARM_FLAG_MODE;
 	cpu->cd.arm.cpsr |= arm_exception_to_mode[exception_nr];
 
+	/*
+	 *  Usually, an exception should change modes (so that saved status
+	 *  bits don't get lost). However, Linux on ARM seems to use floating
+	 *  point instructions in the kernel (!), and it emulates those using
+	 *  its own fp emulation code. This leads to a situation where we
+	 *  sometimes change from SVC32 to SVC32.
+	 */
 	newmode = cpu->cd.arm.cpsr & ARM_FLAG_MODE;
-	if (oldmode == newmode) {
-		fatal("Exception caused no mode change? TODO\n");
-		exit(1);
+	if (oldmode == newmode && oldmode != ARM_MODE_SVC32) {
+		fatal("[ WARNING! Exception caused no mode change? "
+		    "mode 0x%02x (pc=0x%x) ]\n", newmode, (int)cpu->pc);
+		/*  exit(1);  */
 	}
-
+#if 0
+if (oldmode==0x10 && newmode ==0x17 && cpu->pc == 0x1644f0)
+single_step = 1;
+/* 00008554 */
+#endif
 	cpu->cd.arm.cpsr |= ARM_FLAG_I;
 	if (exception_nr == ARM_EXCEPTION_RESET ||
 	    exception_nr == ARM_EXCEPTION_FIQ)
@@ -613,12 +665,18 @@ void arm_exception(struct cpu *cpu, int exception_nr)
 	/*  Load the new register bank, if we switched:  */
 	arm_load_register_bank(cpu);
 
-	/*  Set the return address and new PC:  */
+	/*
+	 *  Set the return address and new PC.
+	 *
+	 *  NOTE: r[ARM_PC] is also set; see cpu_arm_instr_loadstore.c for
+	 *  details. (If an exception occurs during a load into the pc
+	 *  register, the code in that file assumes that the r[ARM_PC]
+	 *  was changed to the address of the exception handler.)
+	 */
 	cpu->cd.arm.r[ARM_LR] = retaddr;
-
 	cpu->pc = cpu->cd.arm.r[ARM_PC] = exception_nr * 4 +
 	    ((cpu->cd.arm.control & ARM_CONTROL_V)? 0xffff0000 : 0);
-	arm_pc_to_pointers(cpu);
+	quick_pc_to_pointers(cpu);
 }
 
 
@@ -632,7 +690,7 @@ void arm_exception(struct cpu *cpu, int exception_nr)
  */
 int arm_cpu_interrupt(struct cpu *cpu, uint64_t irq_nr)
 {
-	/*  fatal("arm_cpu_interrupt(): 0x%llx\n", (int)irq_nr);  */
+	/*  fatal("arm_cpu_interrupt(): 0x%x\n", (int)irq_nr);  */
 	if (irq_nr <= 64) {
 		if (cpu->machine->md_interrupt != NULL)
 			cpu->machine->md_interrupt(cpu->machine,
@@ -752,10 +810,22 @@ int arm_cpu_disassemble_instr(struct cpu *cpu, unsigned char *ib,
 			int a_bit = (iw >> 21) & 1;
 			debug("%s%sl%s%s\t", u_bit? "s" : "u",
 			    a_bit? "mla" : "mul", condition, s_bit? "s" : "");
-			debug("%s,", arm_regname[r12]);
-			debug("%s,", arm_regname[r16]);
-			debug("%s,", arm_regname[iw & 15]);
-			debug("%s\n", arm_regname[r8]);
+			debug("%s,%s,", arm_regname[r12], arm_regname[r16]);
+			debug("%s,%s\n", arm_regname[iw&15], arm_regname[r8]);
+			break;
+		}
+
+		/*
+		 *  xxxx0001 0000nnnn dddd0000 0101mmmm  qadd Rd,Rm,Rn
+		 *  xxxx0001 0010nnnn dddd0000 0101mmmm  qsub Rd,Rm,Rn
+		 *  xxxx0001 0100nnnn dddd0000 0101mmmm  qdadd Rd,Rm,Rn
+		 *  xxxx0001 0110nnnn dddd0000 0101mmmm  qdsub Rd,Rm,Rn
+		 */
+		if ((iw & 0x0f900ff0) == 0x01000050) {
+			debug("q%s%s%s\t", iw & 0x400000? "d" : "",
+			    iw & 0x200000? "sub" : "add", condition);
+			debug("%s,%s,%s\n", arm_regname[r12],
+			    arm_regname[iw&15], arm_regname[r16]);
 			break;
 		}
 
@@ -812,10 +882,69 @@ int arm_cpu_disassemble_instr(struct cpu *cpu, unsigned char *ib,
 			break;
 		}
 
+		/*
+		 *  xxxx0001 01101111 dddd1111 0001mmmm    CLZ Rd,Rm
+		 */
+		if ((iw & 0x0fff0ff0) == 0x016f0f10) {
+			debug("clz%s\t", condition);
+			debug("%s,%s\n", arm_regname[r12], arm_regname[iw&15]);
+			break;
+		}
+
+		/*
+		 *  xxxx0001 0000dddd nnnnssss 1yx0mmmm  SMLAxy Rd,Rm,Rs,Rn
+		 *  xxxx0001 0100dddd DDDDssss 1yx0mmmm  SMLALxy RdL,RdH,Rm,Rs
+		 *  xxxx0001 0010dddd nnnnssss 1y00mmmm  SMLAWy Rd,Rm,Rs,Rn
+		 *  xxxx0001 0110dddd 0000ssss 1yx0mmmm  SMULxy Rd,Rm,Rs
+		 *  xxxx0001 0010dddd 0000ssss 1y10mmmm  SMULWy Rd,Rm,Rs
+		 */
+		if ((iw & 0x0ff00090) == 0x01000080) {
+			debug("smla%s%s%s\t",
+			    iw & 0x20? "t" : "b", iw & 0x40? "t" : "b",
+			    condition);
+			debug("%s,%s,%s,%s\n", arm_regname[r16],
+			    arm_regname[iw&15], arm_regname[r8],
+			    arm_regname[r12]);
+			break;
+		}
+		if ((iw & 0x0ff00090) == 0x01400080) {
+			debug("smlal%s%s%s\t",
+			    iw & 0x20? "t" : "b", iw & 0x40? "t" : "b",
+			    condition);
+			debug("%s,%s,%s,%s\n", arm_regname[r12],
+			    arm_regname[r16], arm_regname[iw&15],
+			    arm_regname[r8]);
+			break;
+		}
+		if ((iw & 0x0ff000b0) == 0x01200080) {
+			debug("smlaw%s%s\t", iw & 0x40? "t" : "b",
+			    condition);
+			debug("%s,%s,%s,%s\n", arm_regname[r16],
+			    arm_regname[iw&15], arm_regname[r8],
+			    arm_regname[r12]);
+			break;
+		}
+		if ((iw & 0x0ff0f090) == 0x01600080) {
+			debug("smul%s%s%s\t",
+			    iw & 0x20? "t" : "b", iw & 0x40? "t" : "b",
+			    condition);
+			debug("%s,%s,%s\n", arm_regname[r16],
+			    arm_regname[iw&15], arm_regname[r8]);
+			break;
+		}
+		if ((iw & 0x0ff0f0b0) == 0x012000a0) {
+			debug("smulw%s%s\t", iw & 0x40? "t" : "b",
+			    condition);
+			debug("%s,%s,%s\n", arm_regname[r16],
+			    arm_regname[iw&15], arm_regname[r8]);
+			break;
+		}
+
 		/*
 		 *  xxxx000P U1WLnnnn ddddHHHH 1SH1LLLL load/store rd,imm(rn)
 		 */
 		if ((iw & 0x0e000090) == 0x00000090) {
+			char *op = "st";
 			int imm = ((iw >> 4) & 0xf0) | (iw & 0xf);
 			int regform = !(iw & 0x00400000);
 			p_bit = main_opcode & 1;
@@ -828,8 +957,11 @@ int arm_cpu_disassemble_instr(struct cpu *cpu, unsigned char *ib,
 				break;
 			}
 			/*  Semi-generic case:  */
-			debug("%sr%s", iw & 0x00100000? "ld" : "st",
-			    condition);
+			if (iw & 0x00100000)
+				op = "ld";
+			if (!l_bit && (iw & 0xd0) == 0xd0)
+				op = iw & 0x20? "st" : "ld";
+			debug("%sr%s", op, condition);
 			if (!l_bit && (iw & 0xd0) == 0xd0) {
 				debug("d");		/*  Double-register  */
 			} else {
@@ -1029,7 +1161,7 @@ int arm_cpu_disassemble_instr(struct cpu *cpu, unsigned char *ib,
 				} else {
 					tmpw[0] = addr = cpu->cd.arm.r[r12];
 					if (r12 == ARM_PC)
-						addr += 8;
+						addr = cpu->pc + 8;
 				}
 				debug(": ");
 				if (b_bit)
@@ -1093,6 +1225,18 @@ int arm_cpu_disassemble_instr(struct cpu *cpu, unsigned char *ib,
 		break;
 	case 0xc:				/*  Coprocessor  */
 	case 0xd:				/*  LDC/STC  */
+		/*
+		 *  xxxx1100 0100nnnn ddddcccc oooommmm    MCRR c,op,Rd,Rn,CRm
+		 *  xxxx1100 0101nnnn ddddcccc oooommmm    MRRC c,op,Rd,Rn,CRm
+		 */
+		if ((iw & 0x0fe00000) == 0x0c400000) {
+			debug("%s%s\t", iw & 0x100000? "mrrc" : "mcrr",
+			    condition);
+			debug("%i,%i,%s,%s,cr%i\n", r8, (iw >> 4) & 15,
+			    arm_regname[r12], arm_regname[r16], iw & 15);
+			break;
+		}
+
 		/*  xxxx110P UNWLnnnn DDDDpppp oooooooo LDC/STC  */
 		debug("TODO: coprocessor LDC/STC\n");
 		break;
@@ -1157,9 +1301,10 @@ void arm_mcr_mrc(struct cpu *cpu, uint32_t iword)
 		cpu->cd.arm.coproc[cp_num](cpu, opcode1, opcode2, l_bit,
 		    crn, crm, rd);
 	else {
-		fatal("arm_mcr_mrc: pc=0x%08x, iword=0x%08x: "
-		    "cp_num=%i\n", (int)cpu->pc, iword, cp_num);
-		exit(1);
+		fatal("[ arm_mcr_mrc: pc=0x%08x, iword=0x%08x: "
+		    "cp_num=%i ]\n", (int)cpu->pc, iword, cp_num);
+		arm_exception(cpu, ARM_EXCEPTION_UND);
+		/*  exit(1);  */
 	}
 }
 
@@ -1174,8 +1319,9 @@ void arm_mcr_mrc(struct cpu *cpu, uint32_t iword)
  */
 void arm_cdp(struct cpu *cpu, uint32_t iword)
 {
-	fatal("arm_cdp: pc=0x%08x, iword=0x%08x\n", (int)cpu->pc, iword);
-	exit(1);
+	fatal("[ arm_cdp: pc=0x%08x, iword=0x%08x ]\n", (int)cpu->pc, iword);
+	arm_exception(cpu, ARM_EXCEPTION_UND);
+	/*  exit(1);  */
 }
 
 
