@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: debugger.c,v 1.122 2005/08/11 09:43:40 debug Exp $
+ *  $Id: debugger.c,v 1.125 2005/11/13 00:14:06 debug Exp $
  *
  *  Single-step debugger.
  *
@@ -180,7 +180,7 @@ void debugger_activate(int x)
  *  Some examples:
  *
  *	"0x7fff1234"		==> numeric value (hex, in this case)
- *	"pc", "r5", "hi", "t4"	==> register (CPU dependant)
+ *	"pc", "r5", "hi", "t4"	==> register (CPU dependent)
  *	"memcpy+64"		==> symbol (plus offset)
  *
  *  Register names can be preceeded by "x:" where x is the CPU number. (CPU
@@ -556,9 +556,9 @@ static void debugger_cmd_device(struct machine *m, char *cmd_line)
 			    (long long)mem->dev_length[i]);
 			if (mem->dev_flags[i]) {
 				printf(" (");
-				if (mem->dev_flags[i] & MEM_DYNTRANS_OK)
+				if (mem->dev_flags[i] & DM_DYNTRANS_OK)
 					printf("DYNTRANS R");
-				if (mem->dev_flags[i] & MEM_DYNTRANS_WRITE_OK)
+				if (mem->dev_flags[i] & DM_DYNTRANS_WRITE_OK)
 					printf("+W");
 				printf(")");
 			}
@@ -1378,7 +1378,7 @@ static void debugger_cmd_tlbdump(struct machine *m, char *cmd_line)
  */
 static void debugger_cmd_trace(struct machine *m, char *cmd_line)
 {
-	int i, toggle = 1;
+	int toggle = 1;
 	int previous_mode = old_show_trace_tree;
 
 	if (cmd_line[0] != '\0') {
@@ -1422,11 +1422,6 @@ static void debugger_cmd_trace(struct machine *m, char *cmd_line)
 	if (m->bintrans_enable && old_show_trace_tree)
 		printf("NOTE: the trace tree functionality doesn't "
 		    "work very well with bintrans!\n");
-
-	/*  Clear translations:  */
-	for (i=0; i<m->ncpus; i++)
-		if (m->cpus[i]->translation_cache != NULL)
-			cpu_create_or_reset_tc(m->cpus[i]);
 }
 
 
@@ -2106,6 +2101,14 @@ void debugger(void)
 		return;
 	}
 
+	/*
+	 *  Clear all dyntrans translations, because otherwise things would
+	 *  become to complex to keep in sync.
+	 */
+	for (i=0; i<debugger_machine->ncpus; i++)
+		if (debugger_machine->cpus[i]->translation_cache != NULL)
+			cpu_create_or_reset_tc(debugger_machine->cpus[i]);
+
 	exit_debugger = 0;
 
 	while (!exit_debugger) {
