@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_bt459.c,v 1.59 2005/02/22 12:15:29 debug Exp $
+ *  $Id: dev_bt459.c,v 1.61 2005/11/13 00:14:08 debug Exp $
  *  
  *  Brooktree 459 vdac, used by TURBOchannel graphics cards.
  */
@@ -357,7 +357,8 @@ int dev_bt459_irq_access(struct cpu *cpu, struct memory *mem,
 	struct bt459_data *d = (struct bt459_data *) extra;
 	uint64_t idata = 0, odata = 0;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 #ifdef BT459_DEBUG
 	fatal("[ bt459: IRQ ack ]\n");
@@ -648,11 +649,11 @@ void dev_bt459_init(struct machine *machine, struct memory *mem,
 	d->interrupt_time_reset_value = 500;
 
 	memory_device_register(mem, "bt459", baseaddr, DEV_BT459_LENGTH,
-	    dev_bt459_access, (void *)d, MEM_DEFAULT, NULL);
+	    dev_bt459_access, (void *)d, DM_DEFAULT, NULL);
 
 	if (baseaddr_irq != 0)
 		memory_device_register(mem, "bt459_irq", baseaddr_irq, 0x10000,
-		    dev_bt459_irq_access, (void *)d, MEM_DEFAULT, NULL);
+		    dev_bt459_irq_access, (void *)d, DM_DEFAULT, NULL);
 
 	machine_add_tickfunction(machine, dev_bt459_tick, d, BT459_TICK_SHIFT);
 
