@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ether.c,v 1.7 2005/08/04 10:53:25 debug Exp $
+ *  $Id: dev_ether.c,v 1.9 2005/11/13 00:14:08 debug Exp $
  *
  *  Basic "ethernet" network device. This is a simple test device which can
  *  be used to send and receive packets to/from a simulated ethernet network.
@@ -112,7 +112,8 @@ int dev_ether_access(struct cpu *cpu, struct memory *mem,
 	unsigned char *incoming_ptr;
 	int incoming_len;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	switch (relative_addr) {
 	case 0x0000:
@@ -229,12 +230,12 @@ int devinit_ether(struct devinit *devinit)
 
 	memory_device_register(devinit->machine->memory, n1,
 	    devinit->addr, DEV_ETHER_MAXBUFLEN, dev_ether_buf_access, (void *)d,
-	    MEM_DYNTRANS_OK | MEM_DYNTRANS_WRITE_OK |
-	    MEM_READING_HAS_NO_SIDE_EFFECTS, d->buf);
+	    DM_DYNTRANS_OK | DM_DYNTRANS_WRITE_OK |
+	    DM_READS_HAVE_NO_SIDE_EFFECTS, d->buf);
 	memory_device_register(devinit->machine->memory, n2,
 	    devinit->addr + DEV_ETHER_MAXBUFLEN,
 	    DEV_ETHER_LENGTH-DEV_ETHER_MAXBUFLEN, dev_ether_access, (void *)d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 
 	net_add_nic(devinit->machine->emul->net, d, d->mac);
 
