@@ -25,14 +25,14 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_fb.c,v 1.107 2005/09/18 19:54:15 debug Exp $
+ *  $Id: dev_fb.c,v 1.110 2005/11/13 00:14:08 debug Exp $
  *  
  *  Generic framebuffer device.
  *
  *	DECstation VFB01 monochrome framebuffer, 1024x864
  *	DECstation VFB02 8-bit color framebuffer, 1024x864
  *	DECstation Maxine, 1024x768 8-bit color
- *	HPCmips framebuffer
+ *	HPC (mips, arm, ..) framebuffer
  *	Playstation 2 (24-bit color)
  *	generic (any resolution, several bit depths possible)
  *
@@ -65,7 +65,7 @@
 #endif
 
 
-#define	FB_TICK_SHIFT		19
+#define	FB_TICK_SHIFT		18
 
 
 /*  #define FB_DEBUG  */
@@ -388,8 +388,8 @@ void update_framebuffer(struct vfb_data *d, int addr, int len)
 				c = d->framebuffer[fb_addr >> 3];
 				fb_addr &= 7;
 
-				/*  HPCmips is reverse:  */
-				if (d->vfb_type == VFB_HPCMIPS)
+				/*  HPC is reverse:  */
+				if (d->vfb_type == VFB_HPC)
 					fb_addr = 8 - d->bit_depth - fb_addr;
 
 				c = (c >> fb_addr) & ((1<<d->bit_depth) - 1);
@@ -435,7 +435,7 @@ void update_framebuffer(struct vfb_data *d, int addr, int len)
 					break;
 				/*  TODO: copy to the scaledown code below  */
 				case 16:
-					if (d->vfb_type == VFB_HPCMIPS) {
+					if (d->vfb_type == VFB_HPC) {
 						b = d->framebuffer[fb_addr] +
 						    (d->framebuffer[fb_addr+1]
 						    << 8);
@@ -521,8 +521,8 @@ void update_framebuffer(struct vfb_data *d, int addr, int len)
 				c = d->framebuffer[fb_addr >> 3];
 				fb_addr &= 7;
 
-				/*  HPCmips is reverse:  */
-				if (d->vfb_type == VFB_HPCMIPS)
+				/*  HPC is reverse:  */
+				if (d->vfb_type == VFB_HPC)
 					fb_addr = 8 - d->bit_depth - fb_addr;
 
 				c = (c >> fb_addr) & ((1<<d->bit_depth) - 1);
@@ -936,7 +936,7 @@ int dev_fb_access(struct cpu *cpu, struct memory *mem,
  *
  *  VFB_DEC_VFB01, _VFB02, and VFB_DEC_MAXINE are DECstation specific.
  *
- *  If type is VFB_HPCMIPS, then color encoding differs from the generic case.
+ *  If type is VFB_HPC, then color encoding differs from the generic case.
  *
  *  If bit_depth = -15 (note the minus sign), then a special hack is used for
  *  the Playstation Portable's 5-bit R, 5-bit G, 5-bit B.
@@ -1056,11 +1056,11 @@ struct vfb_data *dev_fb_init(struct machine *machine, struct memory *mem,
 	}
 	snprintf(name2, nlen, "fb [%s]", name);
 
-	flags = MEM_DEFAULT;
+	flags = DM_DEFAULT;
 	if ((baseaddr & 0xfff) == 0)
-		flags = MEM_DYNTRANS_OK | MEM_DYNTRANS_WRITE_OK;
+		flags = DM_DYNTRANS_OK | DM_DYNTRANS_WRITE_OK;
 
-	flags |= MEM_READING_HAS_NO_SIDE_EFFECTS;
+	flags |= DM_READS_HAVE_NO_SIDE_EFFECTS;
 
 	memory_device_register(mem, name2, baseaddr, size, dev_fb_access,
 	    d, flags, d->framebuffer);
