@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_kn230.c,v 1.11 2005/02/26 16:53:32 debug Exp $
+ *  $Id: dev_kn230.c,v 1.13 2005/11/13 00:14:09 debug Exp $
  *  
  *  DEC MIPSMATE 5100 (KN230) stuff.
  */
@@ -54,7 +54,8 @@ int dev_kn230_access(struct cpu *cpu, struct memory *mem,
 	struct kn230_csr *d = extra;
 	uint64_t idata = 0, odata = 0;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	switch (relative_addr) {
 	case 0:
@@ -100,7 +101,7 @@ int devinit_kn230(struct devinit *devinit)
 
 	memory_device_register(devinit->machine->memory, devinit->name,
 	    devinit->addr, DEV_KN230_LENGTH, dev_kn230_access, d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 
 	devinit->return_ptr = d;
 
