@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_le.c,v 1.43 2005/07/27 06:57:34 debug Exp $
+ *  $Id: dev_le.c,v 1.46 2005/11/13 00:14:09 debug Exp $
  *  
  *  LANCE ethernet, as used in DECstations.
  *
@@ -658,7 +658,8 @@ int dev_le_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	int i, retval = 1;
 	struct le_data *d = extra;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 #ifdef LE_DEBUG
 	if (writeflag == MEM_WRITE) {
@@ -800,14 +801,6 @@ void dev_le_init(struct machine *machine, struct memory *mem, uint64_t baseaddr,
 	/*  ROM (including the MAC address):  */
 	net_generate_unique_mac(machine, &d->rom[0]);
 
-	/*
-	 *  NOTE:  According to the Lance documentation, the low order bit of
-	 *  a physical MAC address should be clear.  However, NetBSD and
-	 *  Linux drop packets if the _first_ byte's lowest bit is not zero.
-	 */
-	d->rom[0] &= ~1;
-	d->rom[5] &= ~1;
-
 	/*  Copies of the MAC address and a test pattern:  */
 	d->rom[10] = d->rom[21] = d->rom[5];
 	d->rom[11] = d->rom[20] = d->rom[4];
@@ -824,8 +817,8 @@ void dev_le_init(struct machine *machine, struct memory *mem, uint64_t baseaddr,
 
 	memory_device_register(mem, "le_sram", baseaddr,
 	    SRAM_SIZE, dev_le_sram_access, (void *)d,
-	    MEM_DYNTRANS_OK | MEM_DYNTRANS_WRITE_OK
-	    | MEM_READING_HAS_NO_SIDE_EFFECTS, d->sram);
+	    DM_DYNTRANS_OK | DM_DYNTRANS_WRITE_OK
+	    | DM_READS_HAVE_NO_SIDE_EFFECTS, d->sram);
 
 	name2 = malloc(nlen);
 	if (name2 == NULL) {
@@ -836,7 +829,7 @@ void dev_le_init(struct machine *machine, struct memory *mem, uint64_t baseaddr,
 	    d->rom[0], d->rom[1], d->rom[2], d->rom[3], d->rom[4], d->rom[5]);
 
 	memory_device_register(mem, name2, baseaddr + 0x100000,
-	    len - 0x100000, dev_le_access, (void *)d, MEM_DEFAULT, NULL);
+	    len - 0x100000, dev_le_access, (void *)d, DM_DEFAULT, NULL);
 
 	machine_add_tickfunction(machine, dev_le_tick, d, LE_TICK_SHIFT);
 
