@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_malta_lcd.c,v 1.2 2005/07/30 20:01:57 debug Exp $
+ *  $Id: dev_malta_lcd.c,v 1.4 2005/11/13 00:14:09 debug Exp $
  *
  *  Malta (evbmips) LCD thingy. Mostly a dummy device.
  */
@@ -88,7 +88,8 @@ int dev_malta_lcd_access(struct cpu *cpu, struct memory *mem,
 	uint64_t idata = 0, odata = 0;
 	int i;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	switch (relative_addr) {
 	case 0x18:
@@ -138,7 +139,7 @@ int devinit_malta_lcd(struct devinit *devinit)
 
 	memory_device_register(devinit->machine->memory, devinit->name,
 	    devinit->addr, DEV_MALTA_LCD_LENGTH,
-	    dev_malta_lcd_access, (void *)d, MEM_DEFAULT, NULL);
+	    dev_malta_lcd_access, (void *)d, DM_DEFAULT, NULL);
 
 	machine_add_tickfunction(devinit->machine, dev_malta_lcd_tick,
 	    d, MALTA_LCD_TICK_SHIFT);
