@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_mc146818.c,v 1.75 2005/10/09 22:21:31 debug Exp $
+ *  $Id: dev_mc146818.c,v 1.80 2005/11/20 11:28:45 debug Exp $
  *  
  *  MC146818 real-time clock, used by many different machines types.
  *  (DS1687 as used in some other machines is also similar to the MC146818.)
@@ -53,6 +53,7 @@
 
 
 #define	to_bcd(x)	( ((x)/10) * 16 + ((x)%10) )
+#define	from_bcd(x)	( ((x)>>4) * 10 + ((x)&15) )
 
 /*  #define MC146818_DEBUG  */
 
@@ -76,12 +77,29 @@ struct mc_data {
 	int	irq_nr;
 
 	int	previous_second;
+	int	n_seconds_elapsed;
+	int	uip_threshold;
 
 	int	interrupt_every_x_cycles;
 	int	cycles_left_until_interrupt;
+
+	int	ugly_netbsd_prep_hack_done;
+	int	ugly_netbsd_prep_hack_sec;
 };
 
 
+/*
+ *  Ugly hack to fool NetBSD/prep to accept the clock.  (See mcclock_isa_match
+ *  in NetBSD's arch/prep/isa/mcclock_isa.c for details.)
+ */
+#define	NETBSD_HACK_INIT		0
+#define	NETBSD_HACK_FIRST_1		1
+#define	NETBSD_HACK_FIRST_2		2
+#define	NETBSD_HACK_SECOND_1		3
+#define	NETBSD_HACK_SECOND_2		4
+#define	NETBSD_HACK_DONE		5
+
+
 /*
  *  recalc_interrupt_cycle():
  *
@@ -133,9 +151,6 @@ void dev_mc146818_tick(struct cpu *cpu, void *extra)
 {
 	struct mc_data *d = extra;
 
-	if (d == NULL)
-		return;
-
 	recalc_interrupt_cycle(cpu, d);
 
 	if ((d->reg[MC_REGB * 4] & MC_REGB_PIE) &&
@@ -226,6 +241,9 @@ static void mc146818_update_time(struct mc_data *d)
 	 *  Special hacks for emulating the behaviour of various machines:
 	 */
 	switch (d->access_style) {
+	case MC146818_ALGOR:
+		d->reg[4 * MC_YEAR] += 80;
+		break;
 	case MC146818_ARC_NEC:
 		d->reg[4 * MC_YEAR] += (0x18 - 104);
 		break;
@@ -307,6 +325,7 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 
 	/*  Different ways of accessing the registers:  */
 	switch (d->access_style) {
+	case MC146818_ALGOR:
 	case MC146818_CATS:
 	case MC146818_PC_CMOS:
 		if ((relative_addr & 1) == 0x00) {
@@ -351,6 +370,10 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 		 *  should be ignored. It works _almost_ as DEC, if offsets are
 		 *  divided by 0x40.
 		 */
+		break;
+	case MC146818_PMPPC:
+		relative_addr *= 4;
+		break;
 	default:
 		;
 	}
@@ -360,7 +383,7 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 		fatal("[ mc146818: write to addr=0x%04x (len %i): ",
 		    (int)relative_addr, (int)len);
 		for (i=0; i<len; i++)
-			fatal("%02x ", data[i]);
+			fatal("0x%02x ", data[i]);
 		fatal("]\n");
 	}
 #endif
@@ -373,23 +396,29 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 	 *  Linux on at least sgimips and evbmips (Malta) wants the UIP bit
 	 *  in REGA to be updated once a second.
 	 */
-	timet = time(NULL);
-	tmp = gmtime(&timet);
-	d->reg[MC_REGC * 4] &= ~MC_REGC_UF;
+	if (relative_addr == MC_REGA*4 || relative_addr == MC_REGC*4) {
+		timet = time(NULL);
+		tmp = gmtime(&timet);
+		d->reg[MC_REGC * 4] &= ~MC_REGC_UF;
+		if (tmp->tm_sec != d->previous_second) {
+			d->n_seconds_elapsed ++;
+			d->previous_second = tmp->tm_sec;
+		}
+		if (d->n_seconds_elapsed > d->uip_threshold) {
+			d->n_seconds_elapsed = 0;
 
-	if (tmp->tm_sec != d->previous_second) {
-		d->reg[MC_REGA * 4] &= ~MC_REGA_UIP;
+			d->reg[MC_REGA * 4] |= MC_REGA_UIP;
 
-		d->reg[MC_REGC * 4] |= MC_REGC_UF;
-		d->reg[MC_REGC * 4] |= MC_REGC_IRQF;
-		d->previous_second = tmp->tm_sec;
+			d->reg[MC_REGC * 4] |= MC_REGC_UF;
+			d->reg[MC_REGC * 4] |= MC_REGC_IRQF;
 
-		/*  For some reason, some Linux/DECstation KN04 kernels want
-		    the PF (periodic flag) bit set, even though interrupts
-		    are not enabled?  */
-		d->reg[MC_REGC * 4] |= MC_REGC_PF;
-	} else
-		d->reg[MC_REGA * 4] |= MC_REGA_UIP;
+			/*  For some reason, some Linux/DECstation KN04
+			    kernels want the PF (periodic flag) bit set,
+			    even though interrupts are not enabled?  */
+			d->reg[MC_REGC * 4] |= MC_REGC_PF;
+		} else
+			d->reg[MC_REGA * 4] &= ~MC_REGA_UIP;
+	}
 
 	/*  RTC data is in either BCD format or binary:  */
 	if (d->use_bcd)
@@ -439,8 +468,7 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 			case MC_RATE_8_Hz:	d->interrupt_hz = 8;	break;
 			case MC_RATE_4_Hz:	d->interrupt_hz = 4;	break;
 			case MC_RATE_2_Hz:	d->interrupt_hz = 2;	break;
-			default:
-				/*  debug("[ mc146818: unimplemented "
+			default:/*  debug("[ mc146818: unimplemented "
 				    "MC_REGA RS: %i ]\n",
 				    data[0] & MC_REGA_RSMASK);  */
 				;
@@ -506,6 +534,24 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 		case 0x15:
 			break;
 		case 4 * MC_SEC:
+			if (d->ugly_netbsd_prep_hack_done < NETBSD_HACK_DONE) {
+				d->ugly_netbsd_prep_hack_done ++;
+				switch (d->ugly_netbsd_prep_hack_done) {
+				case NETBSD_HACK_FIRST_1:
+					d->ugly_netbsd_prep_hack_sec =
+					    from_bcd(d->reg[relative_addr]);
+					break;
+				case NETBSD_HACK_FIRST_2:
+					d->reg[relative_addr] = to_bcd(
+					    d->ugly_netbsd_prep_hack_sec);
+					break;
+				case NETBSD_HACK_SECOND_1:
+				case NETBSD_HACK_SECOND_2:
+					d->reg[relative_addr] = to_bcd((1 +
+					    d->ugly_netbsd_prep_hack_sec) % 60);
+					break;
+				}
+			}
 		case 4 * MC_MIN:
 		case 4 * MC_HOUR:
 		case 4 * MC_DOW:
@@ -522,16 +568,17 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 			if (d->reg[MC_REGB * 4] & MC_REGB_SET)
 				break;
 
-			mc146818_update_time(d);
+			if (d->ugly_netbsd_prep_hack_done >= NETBSD_HACK_DONE)
+				mc146818_update_time(d);
+			break;
+		case 4 * MC_REGA:
 			break;
 		case 4 * MC_REGC:	/*  Interrupt ack.  */
 			/*  NOTE: Acking is done below, _after_ the
 			    register has been read.  */
 			break;
-		default:
-			debug("[ mc146818: read from relative_addr = "
+		default:debug("[ mc146818: read from relative_addr = "
 			    "%04x ]\n", (int)relative_addr);
-			;
 		}
 
 		data[0] = d->reg[relative_addr];
@@ -549,7 +596,7 @@ int dev_mc146818_access(struct cpu *cpu, struct memory *mem,
 		fatal("[ mc146818: read from addr=0x%04x (len %i): ",
 		    (int)relative_addr, (int)len);
 		for (i=0; i<len; i++)
-			fatal("%02x ", data[i]);
+			fatal("0x%02x ", data[i]);
 		fatal("]\n");
 	}
 #endif
@@ -582,10 +629,19 @@ void dev_mc146818_init(struct machine *machine, struct memory *mem,
 	d->access_style  = access_style;
 	d->addrdiv       = addrdiv;
 
-	/*  Only SGIs and PCs use BCD format (?)  */
 	d->use_bcd = 0;
-	if (access_style == MC146818_SGI || access_style == MC146818_PC_CMOS)
+	switch (access_style) {
+	case MC146818_SGI:
+	case MC146818_PC_CMOS:
+	case MC146818_PMPPC:
 		d->use_bcd = 1;
+	}
+
+	if (machine->machine_type != MACHINE_PREP) {
+		/*  NetBSD/prep has a really ugly clock detection code;
+		    all other machines/OSes don't need this.  */
+		d->ugly_netbsd_prep_hack_done = NETBSD_HACK_DONE;
+	}
 
 	if (access_style == MC146818_DEC) {
 	/*  Station Ethernet Address, on DECstation 3100:  */
@@ -625,9 +681,20 @@ void dev_mc146818_init(struct machine *machine, struct memory *mem,
 		d->reg[0xf8] = 1;
 	}
 
+	/*
+	 *  uip_threshold should ideally be 1, but when Linux polls the UIP bit
+	 *  it looses speed. This hack gives Linux the impression that the cpu
+	 *  is uip_threshold times faster than the slow clock it would
+	 *  otherwise detect.
+	 *
+	 *  TODO:  Find out if this messes up Sprite emulation; if so, then
+	 *         this hack has to be removed.
+	 */
+	d->uip_threshold = 5;
+
 	if (access_style == MC146818_ARC_JAZZ)
 		memory_device_register(mem, "mc146818_jazz", 0x90000070ULL,
-		    1, dev_mc146818_jazz_access, d, MEM_DEFAULT, NULL);
+		    1, dev_mc146818_jazz_access, d, DM_DEFAULT, NULL);
 
 	dev_len = DEV_MC146818_LENGTH;
 	switch (access_style) {
@@ -641,7 +708,7 @@ void dev_mc146818_init(struct machine *machine, struct memory *mem,
 
 	memory_device_register(mem, "mc146818", baseaddr,
 	    dev_len * addrdiv, dev_mc146818_access,
-	    d, MEM_DEFAULT, NULL);
+	    d, DM_DEFAULT, NULL);
 
 	mc146818_update_time(d);
 
