@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_mp.c,v 1.28 2005/09/18 19:54:15 debug Exp $
+ *  $Id: dev_mp.c,v 1.30 2005/11/13 00:14:09 debug Exp $
  *
  *  This is a fake multiprocessor (MP) device. It can be useful for
  *  theoretical experiments, but probably bares no resemblance to any
@@ -70,7 +70,8 @@ int dev_mp_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	int i, which_cpu;
 	uint64_t idata = 0, odata = 0;
 
-        idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+	        idata = memory_readmax64(cpu, data, len);
 
 	/*
 	 *  NOTE: It is up to the user of this device to read or write
@@ -271,7 +272,7 @@ int devinit_mp(struct devinit *devinit)
 	memset(d->ipi, 0, sizeof(int *) * n);
 
 	memory_device_register(devinit->machine->memory, devinit->name,
-	    devinit->addr, DEV_MP_LENGTH, dev_mp_access, d, MEM_DEFAULT, NULL);
+	    devinit->addr, DEV_MP_LENGTH, dev_mp_access, d, DM_DEFAULT, NULL);
 
 	return 1;
 }
