@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ns16550.c,v 1.41 2005/09/30 13:33:01 debug Exp $
+ *  $Id: dev_ns16550.c,v 1.46 2005/11/23 18:16:42 debug Exp $
  *  
  *  NS16550 serial controller.
  *
@@ -62,7 +62,7 @@ struct ns_data {
 
 	unsigned char	reg[DEV_NS16550_LENGTH];
 	unsigned char	fcr;		/*  FIFO control register  */
-
+	int		int_asserted;
 	int		dlab;		/*  Divisor Latch Access bit  */
 	int		divisor;
 
@@ -94,11 +94,15 @@ void dev_ns16550_tick(struct cpu *cpu, void *extra)
 	if (((d->reg[com_ier] & IER_ETXRDY) && (d->reg[com_iir] & IIR_TXRDY)) ||
 	    ((d->reg[com_ier] & IER_ERXRDY) && (d->reg[com_iir] & IIR_RXRDY))) {
 		d->reg[com_iir] &= ~IIR_NOPEND;
-		if (d->reg[com_mcr] & MCR_IENABLE)
+		if (d->reg[com_mcr] & MCR_IENABLE) {
 			cpu_interrupt(cpu, d->irqnr);
+			d->int_asserted = 1;
+		}
 	} else {
 		d->reg[com_iir] |= IIR_NOPEND;
-		cpu_interrupt_ack(cpu, d->irqnr);
+		if (d->int_asserted)
+			cpu_interrupt_ack(cpu, d->irqnr);
+		d->int_asserted = 0;
 	}
 }
 
@@ -114,12 +118,15 @@ int dev_ns16550_access(struct cpu *cpu, struct memory *mem,
 	int i;
 	struct ns_data *d = extra;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
+#if 0
 	/*  The NS16550 should be accessed using byte read/writes:  */
 	if (len != 1)
 		fatal("[ ns16550 (%s): len=%i, idata=0x%16llx! ]\n",
 		    d->name, len, (long long)idata);
+#endif
 
 	/*
 	 *  Always ready to transmit:
@@ -159,10 +166,13 @@ int dev_ns16550_access(struct cpu *cpu, struct memory *mem,
 
 		/*  Read/write of data:  */
 		if (writeflag == MEM_WRITE) {
-			if (d->reg[com_mcr] & MCR_LOOPBACK)
+			if (d->reg[com_mcr] & MCR_LOOPBACK) {
 				console_makeavail(d->console_handle, idata);
-			else
+			} else {
 				console_putchar(d->console_handle, idata);
+				if (console_are_slaves_allowed())
+					d->in_use = 1;
+			}
 			d->reg[com_iir] |= IIR_TXRDY;
 		} else {
 			if (d->in_use)
@@ -355,7 +365,7 @@ int devinit_ns16550(struct devinit *devinit)
 
 	memory_device_register(devinit->machine->memory, name, devinit->addr,
 	    DEV_NS16550_LENGTH * d->addrmult, dev_ns16550_access, d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 	machine_add_tickfunction(devinit->machine,
 	    dev_ns16550_tick, d, TICK_SHIFT);
 
