@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pcic.c,v 1.12 2005/09/11 10:37:38 debug Exp $
+ *  $Id: dev_pcic.c,v 1.14 2005/11/13 00:14:09 debug Exp $
  *
  *  Intel 82365SL PC Card Interface Controller (called "pcic" by NetBSD).
  *
@@ -153,7 +153,8 @@ int dev_pcic_access(struct cpu *cpu, struct memory *mem,
 	uint64_t idata = 0, odata = 0;
 	int controller_nr, socket_nr;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	controller_nr = d->regnr & 0x80? 1 : 0;
 	socket_nr = d->regnr & 0x40? 1 : 0;
@@ -229,12 +230,12 @@ int devinit_pcic(struct devinit *devinit)
 
 	memory_device_register(devinit->machine->memory, devinit->name,
 	    devinit->addr, DEV_PCIC_LENGTH,
-	    dev_pcic_access, (void *)d, MEM_DEFAULT, NULL);
+	    dev_pcic_access, (void *)d, DM_DEFAULT, NULL);
 
 	/*  TODO: this shouldn't be hardcoded for hpcmips here!  */
 	memory_device_register(devinit->machine->memory, "pcic_cis",
 	    0x10070000, 0x1000, dev_pcic_cis_access, (void *)d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 
 	/*  TODO: find out a good way to specify the address, and the IRQ!  */
 	/*  IRQ 8 + 32 + 9  */
