@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pmagja.c,v 1.15 2005/07/19 10:48:06 debug Exp $
+ *  $Id: dev_pmagja.c,v 1.17 2005/11/13 00:14:09 debug Exp $
  *  
  *  TURBOchannel PMAG-JA graphics device.
  *
@@ -75,9 +75,10 @@ int dev_pmagja_access(struct cpu *cpu, struct memory *mem,
 	struct pmagja_data *d = extra;
 	int i, res = 1;
 
-	relative_addr += PMAGJA_FIRSTOFFSET;
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
-	idata = memory_readmax64(cpu, data, len);
+	relative_addr += PMAGJA_FIRSTOFFSET;
 
 #ifdef JA_DEBUG
 {
@@ -235,6 +236,6 @@ void dev_pmagja_init(struct machine *machine, struct memory *mem,
 
 	memory_device_register(mem, "pmagja", baseaddr + PMAGJA_FIRSTOFFSET,
 	    DEV_PMAGJA_LENGTH - PMAGJA_FIRSTOFFSET, dev_pmagja_access, d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 }
 
