@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ps2_gs.c,v 1.16 2005/02/26 11:40:28 debug Exp $
+ *  $Id: dev_ps2_gs.c,v 1.18 2005/11/13 00:14:09 debug Exp $
  *  
  *  Playstation 2 "graphics system".
  */
@@ -95,7 +95,9 @@ int dev_ps2_gs_access(struct cpu *cpu, struct memory *mem,
 	int i, regnr;
 	struct gs_data *d = extra;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
+
 	regnr = relative_addr / 16;
 	if (relative_addr & 0xf) {
 		debug("[ gs unaligned access, addr 0x%x ]\n",
@@ -160,7 +162,7 @@ int devinit_ps2_gs(struct devinit *devinit)
 
 	memory_device_register(devinit->machine->memory, devinit->name,
 	    devinit->addr, DEV_PS2_GS_LENGTH, dev_ps2_gs_access, d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 
 	return 1;
 }
