@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_px.c,v 1.28 2005/07/19 10:48:06 debug Exp $
+ *  $Id: dev_px.c,v 1.30 2005/11/13 00:14:09 debug Exp $
  *  
  *  TURBOchannel Pixelstamp graphics device.
  *
@@ -567,7 +567,8 @@ int dev_px_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	struct px_data *d = extra;
 	int i;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	if (relative_addr < 0x0c0000) {
 		/*
@@ -819,7 +820,7 @@ void dev_px_init(struct machine *machine, struct memory *mem,
 	}
 
 	memory_device_register(mem, "px", baseaddr, DEV_PX_LENGTH,
-	    dev_px_access, d, MEM_DEFAULT, NULL);
+	    dev_px_access, d, DM_DEFAULT, NULL);
 	machine_add_tickfunction(machine, dev_px_tick, d, 14);
 }
 
