@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_rd94.c,v 1.24 2005/03/18 23:20:52 debug Exp $
+ *  $Id: dev_rd94.c,v 1.29 2005/11/21 09:17:27 debug Exp $
  *  
  *  Used by NEC-RD94, -R94, and -R96.
  */
@@ -90,7 +90,9 @@ int dev_rd94_access(struct cpu *cpu, struct memory *mem,
 	uint64_t idata = 0, odata = 0;
 	int regnr;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
+
 	regnr = relative_addr / sizeof(uint32_t);
 
 	switch (relative_addr) {
@@ -175,12 +177,11 @@ int dev_rd94_access(struct cpu *cpu, struct memory *mem,
 		if (writeflag == MEM_WRITE) {
 			bus_pci_access(cpu, mem, relative_addr ==
 			    RD94_SYS_PCI_CONFADDR? BUS_PCI_ADDR : BUS_PCI_DATA,
-			    &idata, writeflag, d->pci_data);
+			    &idata, len, writeflag, d->pci_data);
 		} else {
 			bus_pci_access(cpu, mem, relative_addr ==
 			    RD94_SYS_PCI_CONFADDR? BUS_PCI_ADDR : BUS_PCI_DATA,
-			    &odata, writeflag, d->pci_data);
-			/*  odata = 0;  */
+			    &odata, len, writeflag, d->pci_data);
 		}
 		break;
 	default:
@@ -212,11 +213,11 @@ int devinit_rd94(struct devinit *devinit)
 	}
 	memset(d, 0, sizeof(struct rd94_data));
 	d->pciirq   = devinit->irq_nr;
-	d->pci_data = bus_pci_init(d->pciirq);
+	d->pci_data = bus_pci_init(d->pciirq, 0,0, 0,0,0, 0,0,0);
 
 	memory_device_register(devinit->machine->memory, devinit->name,
 	    devinit->addr, DEV_RD94_LENGTH,
-	    dev_rd94_access, (void *)d, MEM_DEFAULT, NULL);
+	    dev_rd94_access, (void *)d, DM_DEFAULT, NULL);
 
 	machine_add_tickfunction(devinit->machine, dev_rd94_tick,
 	    d, RD94_TICK_SHIFT);
