@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sfb.c,v 1.6 2005/02/18 07:22:30 debug Exp $
+ *  $Id: dev_sfb.c,v 1.8 2005/11/13 00:14:09 debug Exp $
  *  
  *  SFB TURBOchannel framebuffer.
  *
@@ -73,7 +73,8 @@ int dev_sfb_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	uint64_t idata = 0, odata = 0;
 	struct sfb_data *d = extra;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	if (writeflag==MEM_READ) {
 		odata = d->reg[(relative_addr >> 2) & (N_SFB_REGS - 1)];
@@ -113,6 +114,6 @@ void dev_sfb_init(struct machine *machine, struct memory *mem,
 	d->reg[(SFB_VVERTICAL - SFB_ASIC_OFFSET) / 4]   = SFB_YSIZE;
 
 	memory_device_register(mem, "sfb", baseaddr, SFB_REG_SIZE,
-	    dev_sfb_access, d, MEM_DEFAULT, NULL);
+	    dev_sfb_access, d, DM_DEFAULT, NULL);
 }
 
