@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sgi_ip20.c,v 1.10 2005/02/22 05:52:58 debug Exp $
+ *  $Id: dev_sgi_ip20.c,v 1.12 2005/11/13 00:14:09 debug Exp $
  *  
  *  SGI IP20 stuff.
  */
@@ -52,7 +52,8 @@ int dev_sgi_ip20_access(struct cpu *cpu, struct memory *mem,
 	/*  struct sgi_ip20_data *d = (struct sgi_ip20_data *) extra;  */
 	uint64_t idata = 0, odata = 0;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	switch (relative_addr) {
 	case 0x38:
@@ -113,7 +114,7 @@ struct sgi_ip20_data *dev_sgi_ip20_init(struct cpu *cpu, struct memory *mem,
 	 */
 	memory_device_register(mem, "sgi_ip20_int", baseaddr,
 	    DEV_SGI_IP20_LENGTH, dev_sgi_ip20_access, (void *)d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 
 	return d;
 }
