@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sgi_ip30.c,v 1.17 2005/02/25 06:14:30 debug Exp $
+ *  $Id: dev_sgi_ip30.c,v 1.19 2005/11/13 00:14:10 debug Exp $
  *  
  *  SGI IP30 stuff.
  *
@@ -67,7 +67,8 @@ int dev_sgi_ip30_access(struct cpu *cpu, struct memory *mem,
 	struct sgi_ip30_data *d = (struct sgi_ip30_data *) extra;
 	uint64_t idata = 0, odata = 0;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	switch (relative_addr) {
 	case 0x20:
@@ -368,15 +369,15 @@ struct sgi_ip30_data *dev_sgi_ip30_init(struct machine *machine,
 
 	memory_device_register(mem, "sgi_ip30_1", baseaddr,
 	    DEV_SGI_IP30_LENGTH, dev_sgi_ip30_access, (void *)d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 	memory_device_register(mem, "sgi_ip30_2", 0x10000000,
-	    0x10000, dev_sgi_ip30_2_access, (void *)d, MEM_DEFAULT, NULL);
+	    0x10000, dev_sgi_ip30_2_access, (void *)d, DM_DEFAULT, NULL);
 	memory_device_register(mem, "sgi_ip30_3", 0x1f000000,
-	    0x10000, dev_sgi_ip30_3_access, (void *)d, MEM_DEFAULT, NULL);
+	    0x10000, dev_sgi_ip30_3_access, (void *)d, DM_DEFAULT, NULL);
 	memory_device_register(mem, "sgi_ip30_4", 0x1f600000,
-	    0x10000, dev_sgi_ip30_4_access, (void *)d, MEM_DEFAULT, NULL);
+	    0x10000, dev_sgi_ip30_4_access, (void *)d, DM_DEFAULT, NULL);
 	memory_device_register(mem, "sgi_ip30_5", 0x1f6c0000,
-	    0x10000, dev_sgi_ip30_5_access, (void *)d, MEM_DEFAULT, NULL);
+	    0x10000, dev_sgi_ip30_5_access, (void *)d, DM_DEFAULT, NULL);
 
 	machine_add_tickfunction(machine, dev_sgi_ip30_tick, d, 16);
 
