@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sgi_ip32.c,v 1.32 2005/08/19 09:43:35 debug Exp $
+ *  $Id: dev_sgi_ip32.c,v 1.38 2005/11/21 09:17:27 debug Exp $
  *  
  *  SGI IP32 devices.
  *
@@ -110,23 +110,22 @@ int dev_crime_access(struct cpu *cpu, struct memory *mem,
 	uint64_t relative_addr, unsigned char *data, size_t len,
 	int writeflag, void *extra)
 {
-	int i;
 	struct crime_data *d = extra;
-	uint64_t idata;
+	uint64_t idata = 0;
+	int i;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	/*
 	 *  Set crime version/revision:
 	 *
-	 *  This might not be the most elegant or correct solution,
-	 *  but it seems that the IP32 PROM likes 0x11 for machines
-	 *  without graphics, and 0xa1 for machines with graphics.
+	 *  This might not be the most elegant or correct solution, but it
+	 *  seems that the IP32 PROM likes 0x11 for machines without graphics,
+	 *  and 0xa1 for machines with graphics.
 	 *
-	 *  NetBSD 2.0 complains about "unknown" crime for 0x11,
-	 *  but I guess that's something one has to live with.
-	 *
-	 *  (TODO?)
+	 *  NetBSD 2.0 complains about "unknown" crime for 0x11, but I guess
+	 *  that's something one has to live with.  (TODO?)
 	 */
 	d->reg[4] = 0x00; d->reg[5] = 0x00; d->reg[6] = 0x00;
 	d->reg[7] = d->use_fb? 0xa1 : 0x11;
@@ -242,7 +241,7 @@ struct crime_data *dev_crime_init(struct machine *machine, struct memory *mem,
 	d->use_fb = use_fb;
 
 	memory_device_register(mem, "crime", baseaddr, DEV_CRIME_LENGTH,
-	    dev_crime_access, d, MEM_DEFAULT, NULL);
+	    dev_crime_access, d, DM_DEFAULT, NULL);
 	machine_add_tickfunction(machine, dev_crime_tick, d, CRIME_TICKSHIFT);
 
 	return d;
@@ -324,7 +323,7 @@ struct mace_data *dev_mace_init(struct memory *mem, uint64_t baseaddr,
 	d->irqnr = irqnr;
 
 	memory_device_register(mem, "mace", baseaddr, DEV_MACE_LENGTH,
-	    dev_mace_access, d, MEM_DEFAULT, NULL);
+	    dev_mace_access, d, DM_DEFAULT, NULL);
 
 	return d;
 }
@@ -344,7 +343,9 @@ int dev_macepci_access(struct cpu *cpu, struct memory *mem,
 	uint64_t idata = 0, odata=0;
 	int regnr, res = 1;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
+
 	regnr = relative_addr / sizeof(uint32_t);
 
 	/*  Read from/write to the macepci:  */
@@ -371,11 +372,10 @@ int dev_macepci_access(struct cpu *cpu, struct memory *mem,
 	case 0xcfc:	/*  PCI DATA  */
 		if (writeflag == MEM_WRITE) {
 			res = bus_pci_access(cpu, mem, relative_addr,
-			    &idata, writeflag, d->pci_data);
+			    &idata, len, writeflag, d->pci_data);
 		} else {
 			res = bus_pci_access(cpu, mem, relative_addr,
-			    &odata, writeflag, d->pci_data);
-			/*  odata = 0;  */
+			    &odata, len, writeflag, d->pci_data);
 		}
 		break;
 	default:
@@ -409,10 +409,10 @@ struct pci_data *dev_macepci_init(struct memory *mem, uint64_t baseaddr,
 	}
 	memset(d, 0, sizeof(struct macepci_data));
 
-	d->pci_data = bus_pci_init(pciirq);
+	d->pci_data = bus_pci_init(pciirq, 0,0, 0,0,0, 0,0,0);
 
 	memory_device_register(mem, "macepci", baseaddr, DEV_MACEPCI_LENGTH,
-	    dev_macepci_access, (void *)d, MEM_DEFAULT, NULL);
+	    dev_macepci_access, (void *)d, DM_DEFAULT, NULL);
 
 	return d->pci_data;
 }
@@ -775,7 +775,9 @@ int dev_sgi_mec_access(struct cpu *cpu, struct memory *mem,
 	uint64_t idata = 0, odata = 0;
 	int regnr;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
+
 	regnr = relative_addr / sizeof(uint64_t);
 
 	/*  Treat most registers as read/write, by default.  */
@@ -954,7 +956,7 @@ void dev_sgi_mec_init(struct machine *machine, struct memory *mem,
 
 	memory_device_register(mem, name2, baseaddr,
 	    DEV_SGI_MEC_LENGTH, dev_sgi_mec_access, (void *)d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 
 	machine_add_tickfunction(machine, dev_sgi_mec_tick, d, MEC_TICK_SHIFT);
 
@@ -1025,7 +1027,7 @@ void dev_sgi_ust_init(struct memory *mem, uint64_t baseaddr)
 
 	memory_device_register(mem, "sgi_ust", baseaddr,
 	    DEV_SGI_UST_LENGTH, dev_sgi_ust_access, (void *)d,
-	    MEM_DEFAULT, NULL);
+	    DM_DEFAULT, NULL);
 }
 
 
@@ -1300,6 +1302,6 @@ void dev_sgi_mte_init(struct memory *mem, uint64_t baseaddr)
 	memset(d, 0, sizeof(struct sgi_mte_data));
 
 	memory_device_register(mem, "sgi_mte", baseaddr, DEV_SGI_MTE_LENGTH,
-	    dev_sgi_mte_access, (void *)d, MEM_DEFAULT, NULL);
+	    dev_sgi_mte_access, (void *)d, DM_DEFAULT, NULL);
 }
 
