@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_turbochannel.c,v 1.43 2005/07/12 08:49:13 debug Exp $
+ *  $Id: dev_turbochannel.c,v 1.45 2005/11/13 00:14:10 debug Exp $
  *  
  *  Generic framework for TURBOchannel devices, used in DECstation machines.
  */
@@ -73,7 +73,8 @@ int dev_turbochannel_access(struct cpu *cpu, struct memory *mem,
 	struct turbochannel_data *d = extra;
 	uint64_t idata = 0, odata = 0;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
 
 	relative_addr += d->rom_skip;
 
@@ -335,6 +336,6 @@ void dev_turbochannel_init(struct machine *machine, struct memory *mem,
 		snprintf(name2, nlen, "turbochannel");
 
 	memory_device_register(mem, name2, baseaddr + rom_offset + rom_skip,
-	    rom_length-rom_skip, dev_turbochannel_access, d, MEM_DEFAULT, NULL);
+	    rom_length-rom_skip, dev_turbochannel_access, d, DM_DEFAULT, NULL);
 }
 
