@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_vr41xx.c,v 1.30 2005/08/05 09:11:48 debug Exp $
+ *  $Id: dev_vr41xx.c,v 1.33 2005/11/13 00:14:10 debug Exp $
  *  
  *  VR41xx (actually, VR4122 and VR4131) misc functions.
  *
@@ -398,7 +398,9 @@ int dev_vr41xx_access(struct cpu *cpu, struct memory *mem,
 	int regnr;
 	int revision = 0;
 
-	idata = memory_readmax64(cpu, data, len);
+	if (writeflag == MEM_WRITE)
+		idata = memory_readmax64(cpu, data, len);
+
 	regnr = relative_addr / sizeof(uint64_t);
 
 	/*  KIU ("Keyboard Interface Unit") is handled separately.  */
@@ -569,7 +571,7 @@ struct vr41xx_data *dev_vr41xx_init(struct machine *machine,
 		break;
 	case 4181:
 		baseaddr = 0xa000000;
-		dev_ram_init(mem, 0xb000000, 0x1000000, DEV_RAM_MIRROR,
+		dev_ram_init(machine, 0xb000000, 0x1000000, DEV_RAM_MIRROR,
 		    0xa000000);
 		break;
 	case 4122:
@@ -582,7 +584,7 @@ struct vr41xx_data *dev_vr41xx_init(struct machine *machine,
 	}
 
 	memory_device_register(mem, "vr41xx", baseaddr, DEV_VR41XX_LENGTH,
-	    dev_vr41xx_access, (void *)d, MEM_DEFAULT, NULL);
+	    dev_vr41xx_access, (void *)d, DM_DEFAULT, NULL);
 
 	/*
 	 *  TODO: Find out which controllers are at which addresses on
@@ -607,7 +609,8 @@ struct vr41xx_data *dev_vr41xx_init(struct machine *machine,
 
 	/*  Some machines (?) use ISA space at 0x15000000 instead of
 	    0x14000000, eg IBM WorkPad Z50.  */
-	dev_ram_init(mem, 0x15000000, 0x1000000, DEV_RAM_MIRROR, 0x14000000);
+	dev_ram_init(machine, 0x15000000, 0x1000000, DEV_RAM_MIRROR,
+	    0x14000000);
 
 	return d;
 }
