@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: emul.c,v 1.234 2005/09/17 21:55:19 debug Exp $
+ *  $Id: emul.c,v 1.238 2005/11/19 18:53:06 debug Exp $
  *
  *  Emulation startup and misc. routines.
  */
@@ -112,11 +112,14 @@ static void add_dump_points(struct machine *m)
 		 *  were automatically converted into the correct address.
 		 */
 
-		if ((dp >> 32) == 0 && ((dp >> 31) & 1))
-			dp |= 0xffffffff00000000ULL;
+		if (m->arch == ARCH_MIPS) {
+			if ((dp >> 32) == 0 && ((dp >> 31) & 1))
+				dp |= 0xffffffff00000000ULL;
+		}
+
 		m->breakpoint_addr[i] = dp;
 
-		debug("breakpoint %i: 0x%016llx", i, (long long)dp);
+		debug("breakpoint %i: 0x%llx", i, (long long)dp);
 		if (string_flag)
 			debug(" (%s)", m->breakpoint_string[i]);
 		debug("\n");
@@ -650,7 +653,7 @@ static int load_bootblock(struct machine *m, struct cpu *cpu,
 
 	/*
 	 *  Try reading a kernel manually from the disk. The code here
-	 *  does not rely on machine-dependant boot blocks etc.
+	 *  does not rely on machine-dependent boot blocks etc.
 	 */
 	/*  ISO9660: (0x800 bytes at 0x8000)  */
 	bootblock_buf = malloc(0x800);
@@ -1210,13 +1213,16 @@ void emul_machine_setup(struct machine *m, int n_load, char **load_names,
 		switch (m->arch) {
 
 		case ARCH_ALPHA:
-			/*  For position-independant code:  */
+			/*  For position-independent code:  */
 			cpu->cd.alpha.r[ALPHA_T12] = cpu->pc;
 			break;
 
 		case ARCH_ARM:
+			if (cpu->pc & 3) {
+				fatal("ARM: lowest bits of pc set: TODO\n");
+				exit(1);
+			}
 			cpu->pc &= 0xfffffffc;
-			cpu->cd.arm.r[ARM_PC] = cpu->pc;
 			break;
 
 		case ARCH_AVR:
@@ -1240,8 +1246,7 @@ void emul_machine_setup(struct machine *m, int n_load, char **load_names,
 			break;
 
 		case ARCH_MIPS:
-			if ((cpu->pc >> 32) == 0
-			    && (cpu->pc & 0x80000000ULL))
+			if ((cpu->pc >> 32) == 0 && (cpu->pc & 0x80000000ULL))
 				cpu->pc |= 0xffffffff00000000ULL;
 
 			cpu->cd.mips.gpr[MIPS_GPR_GP] = gp;
@@ -1352,6 +1357,15 @@ void emul_machine_setup(struct machine *m, int n_load, char **load_names,
 	    m->machine_type == MACHINE_SGI) && m->prom_emulation)
 		add_arc_components(m);
 
+
+#if 0
+if (m->machine_type == MACHINE_IQ80321) {
+	store_32bit_word(cpu, 0xc0200000, 0);
+	store_32bit_word(cpu, 0xc0200004, 0xd0000000);
+}
+#endif
+
+
 	debug("starting cpu%i at ", m->bootstrap_cpu);
 	switch (m->arch) {
 
