@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu.h,v 1.48 2005/09/17 17:14:28 debug Exp $
+ *  $Id: cpu.h,v 1.54 2005/11/16 21:15:19 debug Exp $
  *
  *  See cpu.c.
  */
@@ -49,6 +49,7 @@
 #include "cpu_ia64.h"
 #include "cpu_m68k.h"
 #include "cpu_mips.h"
+#include "cpu_newmips.h"
 #include "cpu_ppc.h"
 #include "cpu_sh.h"
 #include "cpu_sparc.h"
@@ -107,7 +108,7 @@ struct cpu_family {
 #define	TRANSLATIONS			1
 #define	COMBINATIONS			2
 
-#define	DYNTRANS_CACHE_SIZE		(16*1048576)
+#define	DYNTRANS_CACHE_SIZE		(20*1048576)
 #define	DYNTRANS_CACHE_MARGIN		300000
 
 #define	N_BASE_TABLE_ENTRIES		32768
@@ -140,7 +141,7 @@ struct cpu {
 	void		(*update_translation_table)(struct cpu *,
 			    uint64_t vaddr_page, unsigned char *host_page,
 			    int writeflag, uint64_t paddr_page);
-	void		(*invalidate_translation_caches_paddr)(struct cpu *,
+	void		(*invalidate_translation_caches)(struct cpu *,
 			    uint64_t paddr, int flags);
 	void		(*invalidate_code_translation)(struct cpu *,
 			    uint64_t paddr, int flags);
@@ -176,6 +177,7 @@ struct cpu {
 		struct ia64_cpu    ia64;
 		struct m68k_cpu    m68k;
 		struct mips_cpu    mips;
+		struct newmips_cpu newmips;
 		struct ppc_cpu     ppc;
 		struct sh_cpu      sh;
 		struct sparc_cpu   sparc;
@@ -215,8 +217,27 @@ void cpu_init(void);
 #define	INVALIDATE_PADDR		4
 #define	INVALIDATE_VADDR		8
 
+#define	TLB_CODE			0x02
+
 
 #define CPU_FAMILY_INIT(n,s)	int n ## _cpu_family_init(		\
+	struct cpu_family *fp) {					\
+	/*  Fill in the cpu_family struct with valid data for this arch.  */ \
+	fp->name = s;							\
+	fp->cpu_new = n ## _cpu_new;					\
+	fp->list_available_types = n ## _cpu_list_available_types;	\
+	fp->register_match = n ## _cpu_register_match;			\
+	fp->disassemble_instr = n ## _cpu_disassemble_instr;		\
+	fp->register_dump = n ## _cpu_register_dump;			\
+	fp->run = n ## _cpu_run;					\
+	fp->dumpinfo = n ## _cpu_dumpinfo;				\
+	fp->interrupt = n ## _cpu_interrupt; 				\
+	fp->interrupt_ack = n ## _cpu_interrupt_ack;			\
+	fp->functioncall_trace = n ## _cpu_functioncall_trace;		\
+	return 1;							\
+	}
+
+#define CPU_OLD_FAMILY_INIT(n,s)	int n ## _cpu_family_init(	\
 	struct cpu_family *fp) {					\
 	/*  Fill in the cpu_family struct with valid data for this arch.  */ \
 	fp->name = s;							\
