@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_alpha.h,v 1.23 2005/08/28 20:16:24 debug Exp $
+ *  $Id: cpu_alpha.h,v 1.27 2005/11/16 21:15:19 debug Exp $
  */
 
 #include "misc.h"
@@ -72,10 +72,10 @@ struct alpha_instr_call {
 
 /*  Translation cache struct for each physical page:  */
 struct alpha_tc_physpage {
+	struct alpha_instr_call ics[ALPHA_IC_ENTRIES_PER_PAGE + 1];
 	uint32_t	next_ofs;	/*  or 0 for end of chain  */
 	uint32_t	physaddr;
 	int		flags;
-	struct alpha_instr_call ics[ALPHA_IC_ENTRIES_PER_PAGE + 1];
 };
 
 
@@ -104,12 +104,12 @@ struct alpha_vph_page {
 
 #define	ALPHA_MAX_VPH_TLB_ENTRIES	128
 struct alpha_vpg_tlb_entry {
-	int		valid;
-	int		writeflag;
+	unsigned char	valid;
+	unsigned char	writeflag;
 	int64_t		timestamp;
-	unsigned char	*host_page;
 	uint64_t	vaddr_page;
 	uint64_t	paddr_page;
+	unsigned char	*host_page;
 };
 
 struct alpha_cpu {
@@ -139,6 +139,8 @@ struct alpha_cpu {
 	struct alpha_instr_call	*cur_ic_page;
 	struct alpha_instr_call	*next_ic;
 
+	void			(*combination_check)(struct cpu *,
+				    struct alpha_instr_call *, int low_addr);
 
 	/*
 	 *  Virtual -> physical -> host address translation:
@@ -156,7 +158,7 @@ struct alpha_cpu {
 /*  cpu_alpha.c:  */
 void alpha_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
-void alpha_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t, int);
+void alpha_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void alpha_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 int alpha_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
