@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_arm.h,v 1.44 2005/10/10 18:43:37 debug Exp $
+ *  $Id: cpu_arm.h,v 1.57 2005/11/16 21:15:19 debug Exp $
  */
 
 #include "misc.h"
@@ -85,13 +85,18 @@ struct arm_instr_call {
 
 /*  Translation cache struct for each physical page:  */
 struct arm_tc_physpage {
+	struct arm_instr_call ics[ARM_IC_ENTRIES_PER_PAGE + 1];
 	uint32_t	next_ofs;	/*  or 0 for end of chain  */
 	uint32_t	physaddr;
 	int		flags;
-	struct arm_instr_call ics[ARM_IC_ENTRIES_PER_PAGE + 1];
 };
 
 
+#define	ARM_F_N		8	/*  Same as ARM_FLAG_*, but        */
+#define	ARM_F_Z		4	/*  for the 'flags' field instead  */
+#define	ARM_F_C		2	/*  of cpsr.                       */
+#define	ARM_F_V		1
+
 #define	ARM_FLAG_N	0x80000000	/*  Negative flag  */
 #define	ARM_FLAG_Z	0x40000000	/*  Zero flag  */
 #define	ARM_FLAG_C	0x20000000	/*  Carry flag  */
@@ -132,14 +137,13 @@ struct arm_tc_physpage {
 
 #define	ARM_N_VPH_ENTRIES	1048576
 
-#define	ARM_MAX_VPH_TLB_ENTRIES		32
+#define	ARM_MAX_VPH_TLB_ENTRIES		128
 struct arm_vpg_tlb_entry {
-	int		valid;
-	int		writeflag;
-	int64_t		timestamp;
-	unsigned char	*host_page;
+	unsigned char	valid;
+	unsigned char	writeflag;
 	uint32_t	vaddr_page;
 	uint32_t	paddr_page;
+	unsigned char	*host_page;
 };
 
 
@@ -173,7 +177,14 @@ struct arm_cpu {
 
 	uint32_t		tmp_pc;		/*  Used for load/stores  */
 
-	/*  Flag/status registers:  */
+	/*
+	 *  Flag/status registers:
+	 *
+	 *  NOTE: 'flags' just contains the 4 flag bits. When cpsr is read,
+	 *  the flags should be copied from 'flags', and when cpsr is written
+	 *  to, 'flags' should be updated as well.
+	 */
+	size_t			flags;
 	uint32_t		cpsr;
 	uint32_t		spsr_svc;
 	uint32_t		spsr_abt;
@@ -192,6 +203,10 @@ struct arm_cpu {
 	uint32_t		far;		/*  Fault Address Register  */
 	uint32_t		pid;		/*  Process Id Register  */
 
+	/*  For caching the host address of the L1 translation table:  */
+	unsigned char		*translation_table;
+	uint32_t		last_ttb;
+
 
 	/*
 	 *  Interrupts:
@@ -210,6 +225,8 @@ struct arm_cpu {
 	struct arm_instr_call	*cur_ic_page;
 	struct arm_instr_call	*next_ic;
 
+	void			(*combination_check)(struct cpu *,
+				    struct arm_instr_call *, int low_addr);
 
 	/*
 	 *  Virtual -> physical -> host address translation:
@@ -224,6 +241,12 @@ struct arm_cpu {
 	unsigned char			*host_store[ARM_N_VPH_ENTRIES];
 	uint32_t			phys_addr[ARM_N_VPH_ENTRIES];
 	struct arm_tc_physpage		*phys_page[ARM_N_VPH_ENTRIES];
+
+	uint32_t			phystranslation[ARM_N_VPH_ENTRIES/32];
+	uint8_t				vaddr_to_tlbindex[ARM_N_VPH_ENTRIES];
+
+	/*  ARM specific: */
+	uint32_t			is_userpage[ARM_N_VPH_ENTRIES/32];
 };
 
 
@@ -245,12 +268,16 @@ struct arm_cpu {
 #define	ARM_CONTROL_L4		0x8000
 
 /*  cpu_arm.c:  */
+void arm_setup_initial_translation_table(struct cpu *cpu, uint32_t ttb_addr);
+void arm_translation_table_set_l1(struct cpu *cpu, uint32_t vaddr,
+	uint32_t paddr);
+void arm_translation_table_set_l1_b(struct cpu *cpu, uint32_t vaddr,
+	uint32_t paddr);
 void arm_exception(struct cpu *, int);
 void arm_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
-void arm_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t, int);
+void arm_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void arm_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
-void arm_setup_initial_translation_table(struct cpu *cpu, uint32_t ttb_addr);
 void arm_load_register_bank(struct cpu *cpu);
 void arm_save_register_bank(struct cpu *cpu);
 int arm_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
@@ -268,5 +295,7 @@ void arm_coproc_i80321_14(struct cpu *cpu, int opcode1, int opcode2, int l_bit,
 /*  memory_arm.c:  */
 int arm_translate_address(struct cpu *cpu, uint64_t vaddr,
 	uint64_t *return_addr, int flags);
+int arm_translate_address_mmu(struct cpu *cpu, uint64_t vaddr,
+	uint64_t *return_addr, int flags);
 
 #endif	/*  CPU_ARM_H  */
