@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_avr.h,v 1.4 2005/09/17 22:34:53 debug Exp $
+ *  $Id: cpu_avr.h,v 1.10 2005/11/16 21:15:19 debug Exp $
  */
 
 #include "misc.h"
@@ -49,15 +49,16 @@ struct cpu_family;
 
 struct avr_instr_call {
 	void	(*f)(struct cpu *, struct avr_instr_call *);
+	int	len;
 	size_t	arg[AVR_N_IC_ARGS];
 };
 
 /*  Translation cache struct for each physical page:  */
 struct avr_tc_physpage {
+	struct avr_instr_call ics[AVR_IC_ENTRIES_PER_PAGE + 1];
 	uint32_t	next_ofs;	/*  or 0 for end of chain  */
 	uint32_t	physaddr;
 	int		flags;
-	struct avr_instr_call ics[AVR_IC_ENTRIES_PER_PAGE + 1];
 };
 
 
@@ -65,12 +66,11 @@ struct avr_tc_physpage {
 
 #define	AVR_MAX_VPH_TLB_ENTRIES		256
 struct avr_vpg_tlb_entry {
-	int		valid;
-	int		writeflag;
-	int64_t		timestamp;
-	unsigned char	*host_page;
+	unsigned char	valid;
+	unsigned char	writeflag;
 	uint32_t	vaddr_page;
 	uint32_t	paddr_page;
+	unsigned char	*host_page;
 };
 
 
@@ -117,6 +117,8 @@ struct avr_cpu {
 	struct avr_instr_call	*cur_ic_page;
 	struct avr_instr_call	*next_ic;
 
+	void			(*combination_check)(struct cpu *,
+				    struct avr_instr_call *, int low_addr);
 
 	/*
 	 *  Virtual -> physical -> host address translation:
@@ -131,13 +133,16 @@ struct avr_cpu {
 	unsigned char			*host_store[AVR_N_VPH_ENTRIES];
 	uint32_t			phys_addr[AVR_N_VPH_ENTRIES];
 	struct avr_tc_physpage		*phys_page[AVR_N_VPH_ENTRIES];
+
+	uint32_t			phystranslation[AVR_N_VPH_ENTRIES/32];
+	uint8_t				vaddr_to_tlbindex[AVR_N_VPH_ENTRIES];
 };
 
 
 /*  cpu_avr.c:  */
 void avr_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
-void avr_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t, int);
+void avr_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void avr_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 int avr_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
