@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_ia64.h,v 1.3 2005/08/28 20:16:24 debug Exp $
+ *  $Id: cpu_ia64.h,v 1.7 2005/11/16 21:15:19 debug Exp $
  */
 
 #include "misc.h"
@@ -53,10 +53,10 @@ struct ia64_instr_call {
 
 /*  Translation cache struct for each physical page:  */
 struct ia64_tc_physpage {
-	uint32_t	next_ofs;	/*  or 0 for end of chain  */
-	uint32_t	physaddr;
-	int		flags;
 	struct ia64_instr_call ics[IA64_IC_ENTRIES_PER_PAGE + 1];
+	uint32_t	next_ofs;	/*  or 0 for end of chain  */
+	int		flags;
+	uint64_t	physaddr;
 };
 
 
@@ -81,14 +81,14 @@ struct ia64_vph_page {
 };
 
 
-#define	IA64_MAX_VPH_TLB_ENTRIES	48
+#define	IA64_MAX_VPH_TLB_ENTRIES	64
 struct ia64_vpg_tlb_entry {
-	int		valid;
-	int		writeflag;
+	unsigned char	valid;
+	unsigned char	writeflag;
 	int64_t		timestamp;
-	unsigned char	*host_page;
 	uint64_t	vaddr_page;
 	uint64_t	paddr_page;
+	unsigned char	*host_page;
 };
 
 struct ia64_cpu {
@@ -107,6 +107,8 @@ struct ia64_cpu {
 	struct ia64_instr_call	*cur_ic_page;
 	struct ia64_instr_call	*next_ic;
 
+	void			(*combination_check)(struct cpu *,
+				    struct ia64_instr_call *, int low_addr);
 
 	/*
 	 *  Virtual -> physical -> host address translation:
@@ -123,7 +125,7 @@ struct ia64_cpu {
 /*  cpu_ia64.c:  */
 void ia64_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
-void ia64_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t, int);
+void ia64_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void ia64_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 int ia64_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
