@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_m68k.h,v 1.3 2005/08/28 20:16:24 debug Exp $
+ *  $Id: cpu_m68k.h,v 1.9 2005/11/16 21:15:19 debug Exp $
  */
 
 #include "misc.h"
@@ -50,15 +50,16 @@ struct cpu_family;
 
 struct m68k_instr_call {
 	void	(*f)(struct cpu *, struct m68k_instr_call *);
+	int	len;
 	size_t	arg[M68K_N_IC_ARGS];
 };
 
 /*  Translation cache struct for each physical page:  */
 struct m68k_tc_physpage {
+	struct m68k_instr_call ics[M68K_IC_ENTRIES_PER_PAGE + 1];
 	uint32_t	next_ofs;	/*  or 0 for end of chain  */
 	uint32_t	physaddr;
 	int		flags;
-	struct m68k_instr_call ics[M68K_IC_ENTRIES_PER_PAGE + 1];
 };
 
 
@@ -66,12 +67,11 @@ struct m68k_tc_physpage {
 
 #define	M68K_MAX_VPH_TLB_ENTRIES		256
 struct m68k_vpg_tlb_entry {
-	int		valid;
-	int		writeflag;
-	int64_t		timestamp;
-	unsigned char	*host_page;
+	uint8_t		valid;
+	uint8_t		writeflag;
 	uint32_t	vaddr_page;
 	uint32_t	paddr_page;
+	unsigned char	*host_page;
 };
 
 
@@ -95,6 +95,8 @@ struct m68k_cpu {
 	struct m68k_instr_call	*cur_ic_page;
 	struct m68k_instr_call	*next_ic;
 
+	void			(*combination_check)(struct cpu *,
+				    struct m68k_instr_call *, int low_addr);
 
 	/*
 	 *  Virtual -> physical -> host address translation:
@@ -109,13 +111,16 @@ struct m68k_cpu {
 	unsigned char			*host_store[M68K_N_VPH_ENTRIES];
 	uint32_t			phys_addr[M68K_N_VPH_ENTRIES];
 	struct m68k_tc_physpage		*phys_page[M68K_N_VPH_ENTRIES];
+
+	uint32_t			phystranslation[M68K_N_VPH_ENTRIES/32];
+	int16_t				vaddr_to_tlbindex[M68K_N_VPH_ENTRIES];
 };
 
 
 /*  cpu_m68k.c:  */
 void m68k_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
-void m68k_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t, int);
+void m68k_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void m68k_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 int m68k_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
