@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sparc.h,v 1.9 2005/08/28 20:16:24 debug Exp $
+ *  $Id: cpu_sparc.h,v 1.15 2005/11/16 21:15:19 debug Exp $
  */
 
 #include "misc.h"
@@ -53,20 +53,20 @@ struct sparc_instr_call {
 
 /*  Translation cache struct for each physical page:  */
 struct sparc_tc_physpage {
-	uint32_t	next_ofs;	/*  or 0 for end of chain  */
-	uint64_t	physaddr;
-	int		flags;
 	struct sparc_instr_call ics[SPARC_IC_ENTRIES_PER_PAGE + 1];
+	uint32_t	next_ofs;	/*  or 0 for end of chain  */
+	int		flags;
+	uint64_t	physaddr;
 };
 
 #define	SPARC_N_VPH_ENTRIES		1048576
 
 #define	SPARC_MAX_VPH_TLB_ENTRIES		256
 struct sparc_vpg_tlb_entry {
-	int		valid;
-	int		writeflag;
-	int64_t		timestamp;
+	uint8_t		valid;
+	uint8_t		writeflag;
 	unsigned char	*host_page;
+	int64_t		timestamp;
 	uint64_t	vaddr_page;
 	uint64_t	paddr_page;
 };
@@ -87,6 +87,8 @@ struct sparc_cpu {
 	struct sparc_instr_call	*cur_ic_page;
 	struct sparc_instr_call	*next_ic;
 
+	void			(*combination_check)(struct cpu *,
+				    struct sparc_instr_call *, int low_addr);
 
 	/*
 	 *  Virtual -> physical -> host address translation:
@@ -101,13 +103,16 @@ struct sparc_cpu {
 	unsigned char		    *host_store[SPARC_N_VPH_ENTRIES];
 	uint32_t		    phys_addr[SPARC_N_VPH_ENTRIES]; 
 	struct sparc_tc_physpage    *phys_page[SPARC_N_VPH_ENTRIES];
+
+	uint32_t		    phystranslation[SPARC_N_VPH_ENTRIES/32];
+	uint8_t			    vaddr_to_tlbindex[SPARC_N_VPH_ENTRIES];
 };
 
 
 /*  cpu_sparc.c:  */
 void sparc_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
-void sparc_invalidate_translation_caches_paddr(struct cpu *cpu, uint64_t, int);
+void sparc_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void sparc_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 int sparc_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
