@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_x86.h,v 1.33 2005/08/25 17:32:21 debug Exp $
+ *  $Id: cpu_x86.h,v 1.39 2005/11/16 21:15:19 debug Exp $
  */
 
 #include "misc.h"
@@ -103,25 +103,26 @@ struct x86_model {
 
 struct x86_instr_call {
 	void	(*f)(struct cpu *, struct x86_instr_call *);
+	int	len;
 	size_t	arg[X86_N_IC_ARGS];
 };
 
 /*  Translation cache struct for each physical page:  */
 struct x86_tc_physpage {
-	uint32_t	next_ofs;	/*  or 0 for end of chain  */
-	uint64_t	physaddr;
-	int		flags;
 	struct x86_instr_call ics[X86_IC_ENTRIES_PER_PAGE + 1];
+	uint32_t	next_ofs;	/*  or 0 for end of chain  */
+	int		flags;
+	uint64_t	physaddr;
 };
 
 #define	X86_N_VPH_ENTRIES		1048576
 
 #define	X86_MAX_VPH_TLB_ENTRIES		256
 struct x86_vpg_tlb_entry {
-	int		valid;
-	int		writeflag;
-	int64_t		timestamp;
+	uint8_t		valid;
+	uint8_t		writeflag;
 	unsigned char	*host_page;
+	int64_t		timestamp;
 	uint64_t	vaddr_page;
 	uint64_t	paddr_page;
 };
@@ -190,6 +191,8 @@ struct x86_cpu {
 	struct x86_instr_call   *cur_ic_page;
 	struct x86_instr_call   *next_ic;
 
+	void			(*combination_check)(struct cpu *,
+				    struct x86_instr_call *, int low_addr);
 
 	/*
 	 *  Virtual -> physical -> host address translation:
@@ -204,6 +207,9 @@ struct x86_cpu {
 	unsigned char                   *host_store[X86_N_VPH_ENTRIES];
 	uint32_t                        phys_addr[X86_N_VPH_ENTRIES];
 	struct x86_tc_physpage          *phys_page[X86_N_VPH_ENTRIES];
+
+	uint32_t			phystranslation[X86_N_VPH_ENTRIES/32];
+	uint8_t				vaddr_to_tlbindex[X86_N_VPH_ENTRIES];
 };
 
 
@@ -327,8 +333,17 @@ void reload_segment_descriptor(struct cpu *cpu, int segnr, int selector,
 int x86_interrupt(struct cpu *cpu, int nr, int errcode);
 int x86_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
+void x86_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
+        unsigned char *host_page, int writeflag, uint64_t paddr_page);
+void x8632_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
+        unsigned char *host_page, int writeflag, uint64_t paddr_page);
+void x86_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
+void x8632_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
+void x86_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
+void x8632_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 int x86_cpu_family_init(struct cpu_family *);
 
+
 /*  memory_x86.c:  */
 int x86_translate_address(struct cpu *cpu, uint64_t vaddr,
 	uint64_t *return_addr, int flags);
