@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: machine.h,v 1.78 2005/09/21 19:10:35 debug Exp $
+ *  $Id: machine.h,v 1.86 2005/11/22 02:07:40 debug Exp $
  */
 
 #include <sys/types.h>
@@ -68,12 +68,14 @@ struct sgi_ip20_data;
 struct sgi_ip22_data;
 struct sgi_ip30_data;
 struct isa_pic_data {
-	struct pic8259_data *pic1;
-	struct pic8259_data *pic2;
+	struct pic8259_data	*pic1;
+	struct pic8259_data	*pic2;
+	int			last_int;
+	int			native_irq;
 };
 
 
-#define	MACHINE_NAME_MAXBUF		200
+#define	MACHINE_NAME_MAXBUF		150
 
 struct machine {
 	/*  Pointer back to the emul struct we are in:  */
@@ -216,11 +218,15 @@ struct machine {
 			struct mace_data *mace_data;
 		} ip32;
 		struct footbridge_data *footbridge_data;
+		struct bebox_data *bebox_data;
+		struct prep_data *prep_data;
+		struct cpc700_data *cpc700_data;
 	} md_int;
 
 	/*  X11/framebuffer stuff:  */
 	int	use_x11;
 	int	x11_scaledown;
+	int	x11_scaleup;
 	int	x11_n_display_names;
 	char	**x11_display_names;
 	int	x11_current_display_name_nr;	/*  updated by x11.c  */
@@ -247,6 +253,7 @@ struct machine {
 #define	ARCH_HPPA		10
 #define	ARCH_I960		11
 #define	ARCH_AVR		12
+#define	ARCH_NEWMIPS		13
 
 /*  MIPS:  */
 #define	MACHINE_BAREMIPS	1000
@@ -262,6 +269,7 @@ struct machine {
 #define	MACHINE_SONYNEWS	1010
 #define	MACHINE_EVBMIPS		1011
 #define	MACHINE_PSP		1012
+#define	MACHINE_ALGOR		1013
 
 /*  PPC:  */
 #define	MACHINE_BAREPPC		2000
@@ -323,6 +331,10 @@ struct machine {
 /*  AVR:  */
 #define	MACHINE_BAREAVR		12000
 
+/*  NEWMIPS:  */
+#define	MACHINE_BARENEWMIPS	13000
+#define	MACHINE_TESTNEWMIPS	13001
+
 /*  Other "pseudo"-machines:  */
 #define	MACHINE_NONE		0
 #define	MACHINE_USERLAND	100000
@@ -378,6 +390,10 @@ struct machine {
 #define	MACHINE_ARC_JAZZ_M700		7
 #define	MACHINE_ARC_NEC_R96		8
 
+/*  Algor:  */
+#define	MACHINE_ALGOR_P4032		4
+#define	MACHINE_ALGOR_P5064		5
+
 /*  EVBMIPS:  */
 #define	MACHINE_EVBMIPS_MALTA		1
 #define	MACHINE_EVBMIPS_MALTA_BE	2
