@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: memory.h,v 1.37 2005/09/22 09:07:00 debug Exp $
+ *  $Id: memory.h,v 1.44 2005/11/22 16:26:38 debug Exp $
  *
  *  Memory controller related functions.
  */
@@ -40,7 +40,7 @@
 
 
 #define	DEFAULT_RAM_IN_MB		32
-#define	MAX_DEVICES			24
+#define	MAX_DEVICES			26
 
 #define	DEVICE_STATE_TYPE_INT		1
 #define	DEVICE_STATE_TYPE_UINT64_T	2
@@ -64,6 +64,7 @@ struct memory {
 
 	const char	*dev_name[MAX_DEVICES];
 	uint64_t	dev_baseaddr[MAX_DEVICES];
+	uint64_t	dev_endaddr[MAX_DEVICES];	/*  after the end!  */
 	uint64_t	dev_length[MAX_DEVICES];
 	int		dev_flags[MAX_DEVICES];
 	void		*dev_extra[MAX_DEVICES];
@@ -74,11 +75,11 @@ struct memory {
 			    int *type, char **namep, void **data, size_t *len);
 	unsigned char	*dev_dyntrans_data[MAX_DEVICES];
 
-	int		dev_dyntrans_alignment;
-
 	uint64_t	dev_dyntrans_write_low[MAX_DEVICES];
 	uint64_t	dev_dyntrans_write_high[MAX_DEVICES];
 
+	int		dev_dyntrans_alignment;
+
 
 	/*
 	 *  NOTE/TODO: This bintrans was for MIPS only. Ugly. :-/
@@ -113,23 +114,9 @@ struct memory {
 #define	BITS_PER_MEMBLOCK	20
 #define	MAX_BITS		40
 
-#define	MEM_READ			0
-#define	MEM_WRITE			1
-
-
-#define	CACHE_DATA			0
-#define	CACHE_INSTRUCTION		1
-#define	CACHE_NONE			2
-
-#define	CACHE_FLAGS_MASK		0x3
-
-#define	NO_EXCEPTIONS			16
-#define	PHYSICAL			32
-#define	NO_SEGMENTATION			64	/*  for X86  */
-#define	MEMORY_USER_ACCESS		128	/*  for ARM, at least  */
-
 
 /*  memory.c:  */
+#define	MEM_PCI_LITTLE_ENDIAN	128
 uint64_t memory_readmax64(struct cpu *cpu, unsigned char *buf, int len);
 void memory_writemax64(struct cpu *cpu, unsigned char *buf, int len,
 	uint64_t data);
@@ -163,14 +150,38 @@ int translate_address_generic(struct cpu *cpu, uint64_t vaddr,
 	uint64_t *return_addr, int flags);
 
 
+/*  Writeflag:  */
+#define	MEM_READ			0
+#define	MEM_WRITE			1
+#define	MEM_DOWNGRADE			128
+
+/*  Misc. flags:  */
+#define	CACHE_DATA			0
+#define	CACHE_INSTRUCTION		1
+#define	CACHE_NONE			2
+#define	CACHE_FLAGS_MASK		0x3
+#define	NO_EXCEPTIONS			16
+#define	PHYSICAL			32
+#define	NO_SEGMENTATION			64	/*  for X86  */
+#define	MEMORY_USER_ACCESS		128	/*  for ARM, at least  */
+
+/*  Dyntrans Memory flags:  */
+#define	DM_DEFAULT				0
+#define	DM_DYNTRANS_OK				1
+#define	DM_DYNTRANS_WRITE_OK			2
+#define	DM_READS_HAVE_NO_SIDE_EFFECTS		4
+#define	DM_EMULATED_RAM				8
+
 #define FLAG_WRITEFLAG          1
 #define FLAG_NOEXCEPTIONS       2
 #define FLAG_INSTR              4
 
 int userland_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache);
-#define	MEMORY_ACCESS_FAILED	0
-#define	MEMORY_ACCESS_OK	1
+#define	MEMORY_ACCESS_FAILED		0
+#define	MEMORY_ACCESS_OK		1
+#define	MEMORY_ACCESS_OK_WRITE		2
+#define	MEMORY_NOT_FULL_PAGE		256
 
 void memory_device_dyntrans_access(struct cpu *, struct memory *mem,
 	void *extra, uint64_t *low, uint64_t *high);
@@ -187,11 +198,4 @@ void memory_device_register(struct memory *mem, const char *,
 	void *extra, int flags, unsigned char *dyntrans_data);
 void memory_device_remove(struct memory *mem, int i);
 
-/*  Bit flags:  */
-#define	MEM_DEFAULT				0
-#define	MEM_DYNTRANS_OK				1
-#define	MEM_DYNTRANS_WRITE_OK			2
-#define	MEM_READING_HAS_NO_SIDE_EFFECTS		4
-
-
 #endif	/*  MEMORY_H  */
