@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: opcodes_ppc.h,v 1.32 2005/09/24 23:44:19 debug Exp $
+ *  $Id: opcodes_ppc.h,v 1.42 2005/11/24 01:15:07 debug Exp $
  *
  *
  *  PPC and POWER opcodes. These are gathered from various sources.
@@ -66,6 +66,7 @@
 #define	PPC_HI6_RLWIMI		0x14
 #define	PPC_HI6_RLWINM		0x15
 
+#define	PPC_HI6_RLWNM		0x17
 #define	PPC_HI6_ORI		0x18
 #define	PPC_HI6_ORIS		0x19
 #define	PPC_HI6_XORI		0x1a
@@ -116,6 +117,7 @@
 #define	  PPC_31_MTSR		  210
 #define	  PPC_31_STDCX_DOT	  214
 #define	  PPC_31_STBX		  215
+#define	  PPC_31_SUBFME		  232
 #define	  PPC_31_ADDME		  234
 #define	  PPC_31_MULLW		  235
 #define	  PPC_31_MTSRIN		  242
@@ -128,9 +130,13 @@
 #define	  PPC_31_LHZUX		  311
 #define	  PPC_31_XOR		  316
 #define	  PPC_31_MFSPR		  339
+#define	  PPC_31_LHAX		  343
+#define	  PPC_31_TLBIA		  370
 #define	  PPC_31_MFTB		  371
+#define	  PPC_31_LHAUX		  375
 #define	  PPC_31_STHX		  407
 #define	  PPC_31_ORC		  412
+#define	  PPC_31_SLBIE		  434
 #define	  PPC_31_STHUX		  439
 #define	  PPC_31_OR		  444
 #define	  PPC_31_DCCCI		  454
@@ -139,24 +145,31 @@
 #define	  PPC_31_DCBI		  470
 #define	  PPC_31_NAND		  476
 #define	  PPC_31_DIVW		  491
+#define	  PPC_31_SLBIA		  498
 #define	  PPC_31_CLI		  502
 #define	  PPC_31_SUBFCO		  520
 #define	  PPC_31_ADDCO		  522
 #define	  PPC_31_LWBRX		  534
+#define	  PPC_31_LFSX		  535
 #define	  PPC_31_SRW		  536
 #define	  PPC_31_SUBFO		  552
 #define	  PPC_31_TLBSYNC	  566
+#define	  PPC_31_MFSR		  595
 #define	  PPC_31_LSWI		  597
 #define	  PPC_31_SYNC		  598
+#define	  PPC_31_LFDX		  599
 #define	  PPC_31_NEGO		  616
 #define	  PPC_31_DCLST		  630
 #define	  PPC_31_SUBFEO		  648
 #define	  PPC_31_ADDEO		  650
 #define	  PPC_31_MFSRIN		  659
 #define	  PPC_31_STWBRX		  662
+#define	  PPC_31_STFSX		  663
 #define	  PPC_31_SUBFZEO	  712
 #define	  PPC_31_ADDZEO		  714
 #define	  PPC_31_STSWI		  725
+#define	  PPC_31_STFDX		  727
+#define	  PPC_31_SUBFMEO	  744
 #define	  PPC_31_ADDMEO		  746
 #define	  PPC_31_MULLWO		  747
 #define	  PPC_31_ADDO		  778
@@ -169,9 +182,11 @@
 #define	  PPC_31_EXTSB		  954
 #define	  PPC_31_ICCCI		  966
 #define	  PPC_31_DIVWUO		  971
+#define	  PPC_31_TLBLD		  978
 #define	  PPC_31_ICBI		  982
 #define	  PPC_31_EXTSW		  986
 #define	  PPC_31_DIVWO		  1003
+#define	  PPC_31_TLBLI		  1010
 #define	  PPC_31_DCBZ		  1014
 #define	PPC_HI6_LWZ		0x20
 #define	PPC_HI6_LWZU		0x21
@@ -189,12 +204,38 @@
 #define	PPC_HI6_STHU		0x2d
 #define	PPC_HI6_LMW		0x2e
 #define	PPC_HI6_STMW		0x2f
+#define	PPC_HI6_LFS		0x30
 
 #define	PPC_HI6_LFD		0x32
 
+#define	PPC_HI6_STFS		0x34
+
 #define	PPC_HI6_STFD		0x36
 
+#define	PPC_HI6_LD		0x3a
+#define	PPC_HI6_59		0x3b
+#define	  PPC_59_FDIVS		  18
+#define	  PPC_59_FSUBS		  20
+#define	  PPC_59_FADDS		  21
+#define	  PPC_59_FMULS		  25
+#define	  PPC_59_FMADDS		  29
+
+#define	PPC_HI6_STD		0x3e
 #define	PPC_HI6_63		0x3f
+#define	  PPC_63_FCMPU		  0
+#define	  PPC_63_FRSP		  12
+#define	  PPC_63_FCTIWZ		  15
+#define	  PPC_63_FDIV		  18
+#define	  PPC_63_FSUB		  20
+#define	  PPC_63_FADD		  21
+#define	  PPC_63_FMUL		  25
+#define	  PPC_63_FMSUB		  28
+#define	  PPC_63_FMADD		  29
+#define	  PPC_63_FNEG		  40
 #define	  PPC_63_FMR		  72
+#define	  PPC_63_FNABS		  136
+#define	  PPC_63_FABS		  264
+#define	  PPC_63_MFFS		  583
+#define	  PPC_63_MTFSF		  711
 
 #endif	/*  OPCODES_PPC_H  */
