@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: main.c,v 1.244 2005/09/07 07:41:29 debug Exp $
+ *  $Id: main.c,v 1.248 2005/11/23 02:17:00 debug Exp $
  */
 
 #include <stdio.h>
@@ -513,6 +513,10 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 			break;
 		case 'Y':
 			m->x11_scaledown = atoi(optarg);
+			if (m->x11_scaledown < -1) {
+				m->x11_scaleup = - m->x11_scaledown;
+				m->x11_scaledown = 1;
+			}
 			if (m->x11_scaledown < 1) {
 				fprintf(stderr, "Invalid scaledown value.\n");
 				exit(1);
@@ -596,7 +600,8 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 
 	if ((m->instruction_trace || m->register_dump || m->show_trace_tree)
 	    && m->bintrans_enable) {
-		fprintf(stderr, "Implicitly turning off bintrans.\n");
+		if (m->arch == ARCH_MIPS)
+			fprintf(stderr, "Implicitly turning off bintrans.\n");
 		m->bintrans_enable = 0;
 	}
 
@@ -646,7 +651,7 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 		exit(1);
 	}
 
-	if (m->bintrans_enable) {
+	if (m->bintrans_enable && m->arch == ARCH_MIPS) {
 		m->speed_tricks = 0;
 		/*  TODO: Print a warning about this?  */
 	}
@@ -803,6 +808,7 @@ int main(int argc, char *argv[])
 	}
 
 	device_set_exit_on_error(0);
+	console_warn_if_slaves_are_needed();
 
 	/*  Run all emulations:  */
 	emul_run(emuls, n_emuls);
