@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: memory_rw.c,v 1.65 2005/10/10 18:43:36 debug Exp $
+ *  $Id: memory_rw.c,v 1.80 2005/11/20 11:28:44 debug Exp $
  *
  *  Generic memory_rw(), with special hacks for specific CPU families.
  *
@@ -48,7 +48,7 @@
  *			a placeholder for data when reading from memory
  *	len		the length of the 'data' buffer
  *	writeflag	set to MEM_READ or MEM_WRITE
- *	cache_flags	CACHE_{NONE,DATA,INSTRUCTION} | other flags
+ *	misc_flags	CACHE_{NONE,DATA,INSTRUCTION} | other flags
  *
  *  If the address indicates access to a memory mapped device, that device'
  *  read/write access function is called.
@@ -66,7 +66,7 @@
  *  (MEMORY_ACCESS_FAILED is 0.)
  */
 int MEMORY_RW(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
-	unsigned char *data, size_t len, int writeflag, int cache_flags)
+	unsigned char *data, size_t len, int writeflag, int misc_flags)
 {
 #ifdef MEM_ALPHA
 	const int offset_mask = 0x1fff;
@@ -85,24 +85,24 @@ int MEMORY_RW(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 #endif
 	int bintrans_device_danger = 0;
 
-	no_exceptions = cache_flags & NO_EXCEPTIONS;
-	cache = cache_flags & CACHE_FLAGS_MASK;
+	no_exceptions = misc_flags & NO_EXCEPTIONS;
+	cache = misc_flags & CACHE_FLAGS_MASK;
 
 #ifdef MEM_X86
 	/*  Real-mode wrap-around:  */
-	if (REAL_MODE && !(cache_flags & PHYSICAL)) {
+	if (REAL_MODE && !(misc_flags & PHYSICAL)) {
 		if ((vaddr & 0xffff) + len > 0x10000) {
 			/*  Do one byte at a time:  */
 			int res = 0, i;
 			for (i=0; i<len; i++)
 				res = MEMORY_RW(cpu, mem, vaddr+i, &data[i], 1,
-				    writeflag, cache_flags);
+				    writeflag, misc_flags);
 			return res;
 		}
 	}
 
 	/*  Crossing a page boundary? Then do one byte at a time:  */
-	if ((vaddr & 0xfff) + len > 0x1000 && !(cache_flags & PHYSICAL)
+	if ((vaddr & 0xfff) + len > 0x1000 && !(misc_flags & PHYSICAL)
 	    && cpu->cd.x86.cr[0] & X86_CR0_PG) {
 		/*  For WRITES: Read ALL BYTES FIRST and write them back!!!
 		    Then do a write of all the new bytes. This is to make sure
@@ -113,17 +113,17 @@ int MEMORY_RW(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 			unsigned char tmp;
 			for (i=0; i<len; i++) {
 				res = MEMORY_RW(cpu, mem, vaddr+i, &tmp, 1,
-				    MEM_READ, cache_flags);
+				    MEM_READ, misc_flags);
 				if (!res)
 					return 0;
 				res = MEMORY_RW(cpu, mem, vaddr+i, &tmp, 1,
-				    MEM_WRITE, cache_flags);
+				    MEM_WRITE, misc_flags);
 				if (!res)
 					return 0;
 			}
 			for (i=0; i<len; i++) {
 				res = MEMORY_RW(cpu, mem, vaddr+i, &data[i], 1,
-				    MEM_WRITE, cache_flags);
+				    MEM_WRITE, misc_flags);
 				if (!res)
 					return 0;
 			}
@@ -131,7 +131,7 @@ int MEMORY_RW(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 			for (i=0; i<len; i++) {
 				/*  Do one byte at a time:  */
 				res = MEMORY_RW(cpu, mem, vaddr+i, &data[i], 1,
-				    writeflag, cache_flags);
+				    writeflag, misc_flags);
 				if (!res) {
 					if (cache == CACHE_INSTRUCTION) {
 						fatal("FAILED instruction "
@@ -184,36 +184,21 @@ int MEMORY_RW(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	}
 #endif	/*  MEM_MIPS  */
 
-	if (cache_flags & PHYSICAL || cpu->translate_address == NULL) {
+	if (misc_flags & PHYSICAL || cpu->translate_address == NULL) {
 		paddr = vaddr;
-
 #ifdef MEM_ALPHA
 		/*  paddr &= 0x1fffffff;  For testalpha  */
 		paddr &= 0x000003ffffffffffULL;
 #endif
-
-#ifdef MEM_IA64
-		/*  For testia64  */
-		paddr &= 0x3fffffff;
-#endif
-
-#ifdef MEM_PPC
-		if (cpu->cd.ppc.bits == 32)
-			paddr &= 0xffffffff;
-#endif
-
-#ifdef MEM_SH
-		paddr &= 0xffffffff;
-#endif
 	} else {
 		ok = cpu->translate_address(cpu, vaddr, &paddr,
 		    (writeflag? FLAG_WRITEFLAG : 0) +
 		    (no_exceptions? FLAG_NOEXCEPTIONS : 0)
 #ifdef MEM_X86
-		    + (cache_flags & NO_SEGMENTATION)
+		    + (misc_flags & NO_SEGMENTATION)
 #endif
 #ifdef MEM_ARM
-		    + (cache_flags & MEMORY_USER_ACCESS)
+		    + (misc_flags & MEMORY_USER_ACCESS)
 #endif
 		    + (cache==CACHE_INSTRUCTION? FLAG_INSTR : 0));
 		/*  If the translation caused an exception, or was invalid in
@@ -226,7 +211,7 @@ int MEMORY_RW(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 
 #ifdef MEM_X86
 	/*  DOS debugging :-)  */
-	if (!quiet_mode && !(cache_flags & PHYSICAL)) {
+	if (!quiet_mode && !(misc_flags & PHYSICAL)) {
 		if (paddr >= 0x400 && paddr <= 0x4ff)
 			debug("{ PC BIOS DATA AREA: %s 0x%x }\n", writeflag ==
 			    MEM_WRITE? "writing to" : "reading from",
@@ -310,8 +295,7 @@ have_paddr:
 		 */
 		for (i=0; i<mem->n_mmapped_devices; i++)
 			if (paddr >= (mem->dev_baseaddr[i] & ~offset_mask) &&
-			    paddr <= ((mem->dev_baseaddr[i] +
-			    mem->dev_length[i] - 1) | offset_mask)) {
+			    paddr <= ((mem->dev_endaddr[i]-1) | offset_mask)) {
 				bintrans_device_danger = 1;
 				break;
 			}
@@ -321,7 +305,7 @@ have_paddr:
 		/*  Scan through all devices:  */
 		do {
 			if (paddr >= mem->dev_baseaddr[i] &&
-			    paddr < mem->dev_baseaddr[i] + mem->dev_length[i]) {
+			    paddr < mem->dev_endaddr[i]) {
 				/*  Found a device, let's access it:  */
 				mem->last_accessed_device = i;
 
@@ -330,10 +314,16 @@ have_paddr:
 					len = mem->dev_length[i] - paddr;
 
 				if (cpu->update_translation_table != NULL &&
-				    mem->dev_flags[i] & MEM_DYNTRANS_OK) {
+				    !(ok & MEMORY_NOT_FULL_PAGE) &&
+				    mem->dev_flags[i] & DM_DYNTRANS_OK) {
 					int wf = writeflag == MEM_WRITE? 1 : 0;
+					unsigned char *host_addr;
 
-					if (writeflag) {
+					if (!(mem->dev_flags[i] &
+					    DM_DYNTRANS_WRITE_OK))
+						wf = 0;
+
+					if (writeflag && wf) {
 						if (paddr < mem->
 						    dev_dyntrans_write_low[i])
 							mem->
@@ -348,20 +338,33 @@ have_paddr:
 							    offset_mask;
 					}
 
-					if (!(mem->dev_flags[i] &
-					    MEM_DYNTRANS_WRITE_OK))
-						wf = 0;
-
+					if (mem->dev_flags[i] &
+					    DM_EMULATED_RAM) {
+						/*  MEM_WRITE to force the page
+						    to be allocated, if it
+						    wasn't already  */
+						uint64_t *pp = (uint64_t *)
+						    mem->dev_dyntrans_data[i];
+						uint64_t p = orig_paddr - *pp;
+						host_addr =
+						    memory_paddr_to_hostaddr(
+						    mem, p, MEM_WRITE)
+						    + (p & ~offset_mask
+						    & ((1 <<
+						    BITS_PER_MEMBLOCK) - 1));
+					} else {
+						host_addr =
+						    mem->dev_dyntrans_data[i] +
+						    (paddr & ~offset_mask);
+					}
 					cpu->update_translation_table(cpu,
-					    vaddr & ~offset_mask,
-					    mem->dev_dyntrans_data[i] +
-					    (paddr & ~offset_mask),
+					    vaddr & ~offset_mask, host_addr,
 					    wf, orig_paddr & ~offset_mask);
 				}
 
 				res = 0;
 				if (!no_exceptions || (mem->dev_flags[i] &
-				    MEM_READING_HAS_NO_SIDE_EFFECTS))
+				    DM_READS_HAVE_NO_SIDE_EFFECTS))
 					res = mem->dev_f[i](cpu, mem, paddr,
 					    data, len, writeflag,
 					    mem->dev_extra[i]);
@@ -370,10 +373,12 @@ have_paddr:
 				if (res == 0)
 					res = -1;
 
+#ifdef MEM_MIPS
 				cpu->cd.mips.instruction_delay +=
 				    ( (abs(res) - 1) *
 				     cpu->cd.mips.cpu_type.instrs_per_cycle );
 #endif
+#endif
 
 #ifndef MEM_X86
 				/*
@@ -410,7 +415,7 @@ have_paddr:
 	switch (cpu->cd.mips.cpu_type.mmu_model) {
 	case MMU3K:
 		/*  if not uncached addess  (TODO: generalize this)  */
-		if (!(cache_flags & PHYSICAL) && cache != CACHE_NONE &&
+		if (!(misc_flags & PHYSICAL) && cache != CACHE_NONE &&
 		    !((vaddr & 0xffffffffULL) >= 0xa0000000ULL &&
 		      (vaddr & 0xffffffffULL) <= 0xbfffffffULL)) {
 			if (memory_cache_R3000(cpu, cache, paddr,
@@ -442,15 +447,11 @@ have_paddr:
 		{
 			if (paddr >= mem->physical_max) {
 				char *symbol;
-				uint64_t old_pc;
 				uint64_t offset;
-
 #ifdef MEM_MIPS
-				old_pc = cpu->cd.mips.pc_last;
+				uint64_t old_pc = cpu->cd.mips.pc_last;
 #else
-				/*  Default instruction size on most
-				    RISC archs is 32 bits:  */
-				old_pc = cpu->pc - sizeof(uint32_t);
+				uint64_t old_pc = cpu->pc;
 #endif
 
 				/*  This allows for example OS kernels to probe
@@ -550,6 +551,14 @@ have_paddr:
 
 	/*
 	 *  Uncached access:
+	 *
+	 *  1)  Translate the physical address to a host address.
+	 *
+	 *  2)  Insert this virtual->physical->host translation into the
+	 *      fast translation arrays (using update_translation_table()).
+	 *
+	 *  3)  If this was a Write, then invalidate any code translations
+	 *      in that page.
 	 */
 	memblock = memory_paddr_to_hostaddr(mem, paddr, writeflag);
 	if (memblock == NULL) {
@@ -561,20 +570,29 @@ have_paddr:
 	offset = paddr & ((1 << BITS_PER_MEMBLOCK) - 1);
 
 	if (cpu->update_translation_table != NULL && !bintrans_device_danger
+#ifndef MEM_MIPS
+/*	    && !(misc_flags & MEMORY_USER_ACCESS)  */
+#ifndef MEM_USERLAND
+	    && !(ok & MEMORY_NOT_FULL_PAGE)
+#endif
+#endif
 	    && !no_exceptions)
 		cpu->update_translation_table(cpu, vaddr & ~offset_mask,
 		    memblock + (offset & ~offset_mask),
-#if 0
-		    cache == CACHE_INSTRUCTION?
-			(writeflag == MEM_WRITE? 1 : 0)
-			: ok - 1,
+		    (misc_flags & MEMORY_USER_ACCESS) |
+#ifndef MEM_MIPS
+		    (cache == CACHE_INSTRUCTION? TLB_CODE : 0) |
+#endif
+#if !defined(MEM_MIPS) && !defined(MEM_USERLAND)
+		    (cache == CACHE_INSTRUCTION?
+			(writeflag == MEM_WRITE? 1 : 0) : ok - 1),
 #else
-		    writeflag == MEM_WRITE? 1 : 0,
+		    (writeflag == MEM_WRITE? 1 : 0),
 #endif
 		    paddr & ~offset_mask);
 
-	if (writeflag == MEM_WRITE &&
-	    cpu->invalidate_code_translation != NULL)
+	/*  Invalidate code translations for the page we are writing to.  */
+	if (writeflag == MEM_WRITE && cpu->invalidate_code_translation != NULL)
 		cpu->invalidate_code_translation(cpu, paddr, INVALIDATE_PADDR);
 
 	if (writeflag == MEM_WRITE) {
