@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: net.c,v 1.80 2005/08/12 05:49:46 debug Exp $
+ *  $Id: net.c,v 1.84 2005/11/25 03:38:00 debug Exp $
  *
  *  Emulated (ethernet / internet) network support.
  *
@@ -172,6 +172,11 @@ void net_generate_unique_mac(struct machine *machine, unsigned char *macbuf)
 	macbuf[4] = 0;
 	macbuf[5] = machine->serial_nr << 4;
 
+	if (macbuf[0] & 1 || macbuf[5] & 1) {
+		fatal("Internal error in net_generate_unique_mac().\n");
+		exit(1);
+	}
+
 	/*  TODO: Remember the mac addresses somewhere?  */
 	machine->nr_of_nics ++;
 }
@@ -1430,9 +1435,10 @@ static void net_arp(struct net *net, void *extra,
 			if (memcmp(packet+24, net->gateway_ipv4_addr, 4) != 0)
 				break;
 
-			lp = net_allocate_packet_link(net, extra, len + 14);
+			lp = net_allocate_packet_link(net, extra, 60 + 14);
 
 			/*  Copy the old packet first:  */
+			memset(lp->data, 0, 60 + 14);
 			memcpy(lp->data + 14, packet, len);
 
 			/*  Add ethernet ARP header:  */
@@ -1453,9 +1459,10 @@ static void net_arp(struct net *net, void *extra,
 
 			break;
 		case 3:		/*  Reverse Request  */
-			lp = net_allocate_packet_link(net, extra, len + 14);
+			lp = net_allocate_packet_link(net, extra, 60 + 14);
 
 			/*  Copy the old packet first:  */
+			memset(lp->data, 0, 60 + 14);
 			memcpy(lp->data + 14, packet, len);
 
 			/*  Add ethernet RARP header:  */
@@ -2062,7 +2069,7 @@ void net_ethernet_tx(struct net *net, void *extra,
 
 	/*  ARP:  */
 	if (packet[12] == 0x08 && packet[13] == 0x06) {
-		if (len != 60)
+		if (len != 42 && len != 60)
 			fatal("[ net_ethernet_tx: WARNING! unusual "
 			    "ARP len (%i) ]\n", len);
 		net_arp(net, extra, packet + 14, len - 14, 0);
