@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: useremul.c,v 1.64 2005/08/27 17:29:06 debug Exp $
+ *  $Id: useremul.c,v 1.66 2005/11/11 13:23:15 debug Exp $
  *
  *  Userland (syscall) emulation.
  *
@@ -847,8 +847,7 @@ static void useremul__netbsd(struct cpu *cpu, uint32_t code)
 	}
 
 	/*
-	 *  NOTE: The following code should not be CPU arch dependant!
-	 *  (TODO)
+	 *  NOTE/TODO: The following code should not be CPU arch dependent!
 	 */
 
 	switch (sysnr) {
@@ -1213,6 +1212,7 @@ static void useremul__netbsd(struct cpu *cpu, uint32_t code)
 		}
 		if (result_high_set)
 			cpu->cd.arm.r[1] = result_high;
+		cpu->cd.arm.flags = cpu->cd.arm.cpsr >> 28;
 		break;
 	case ARCH_MIPS:
 		/*
